/*
 * Copyright (C) 2005 by Linus McCabe, <linus@mccabe.nu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "Breakpoint.h"

#include <wx/arrimpl.cpp> // this is a magic incantation which must be done!
WX_DEFINE_OBJARRAY(BreakpointArray);

char * Breakpoint::TypeNames[] = {"line", "true", "change"};

Breakpoint::Breakpoint(Type type, const wxString& filename, long line)
{
  m_type = type;
  m_filename = filename;
  m_line = line;
}

Breakpoint::Breakpoint(Type type, const wxString& expression, const wxString& filename, const wxString& cls, const wxString& function)
{
  m_type = type;
  m_filename = filename;
  m_class = cls;
  m_function = function;
  m_expression = expression;
  m_line = 0;
}

StringHash Breakpoint::getHash()  const
{
  return CompileHash(
    "type", TypeNames[m_type],
    "filename", m_filename.c_str(),
    "line", wxString::Format(wxT("%ld"), m_line).c_str(),
    "expression", m_expression.c_str(),
    "function", m_function.c_str(),
    "class", m_class.c_str(),
    0
    );
    
}


bool Breakpoint::operator == (Breakpoint bp)
{
  if(bp.getFilename() == m_filename
  && bp.getLine() == m_line
  && bp.getType() == m_type
  && bp.getClass() == m_class
  && bp.getFunction() == m_function
  && bp.getExpression() == m_expression
  )
    return true;
  return false;
  
}

wxString Breakpoint::getTypeName() const
{
  switch(m_type)
  {
    case Breakpoint::Change:
      return _("Change");
    case Breakpoint::True:
      return _("True");
    case Breakpoint::Line:
      return _("Line");
  }
  return _("Unknown");
}
