/*
 * Copyright (C) 2005 by Linus McCabe, <linus@mccabe.nu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef BREAKPOINT_H
#define BREAKPOINT_H

#include <wx/wx.h>
#include "wxClientGlobal.h"

class Breakpoint;
#include <wx/dynarray.h>

WX_DECLARE_OBJARRAY(Breakpoint, BreakpointArray);

class Breakpoint 
{

  public:
    enum Type
    {
      Line = 0,
      True,
      Change
    };
      
  private:

    static char * TypeNames[];
    
    wxString m_filename;
    wxString m_expression;
    wxString m_class;
    wxString m_function;
    wxString m_value;
    long m_line;
    Type m_type;
  
  public:
    // ctors
    Breakpoint (Type type, const wxString& filename, long line);
    Breakpoint (Type type, const wxString& expression, const wxString& filename, const wxString& cls, const wxString& function);
    
    //wxString getKey()const ;
    wxString getFilename() const {return m_filename;}
    wxString getExpression() const {return m_expression;}
    wxString getClass() const {return m_class;}
    wxString getFunction() const {return m_function;}
    wxString getValue() const {return m_value;}
    long getLine() const {return m_line;}
    Type getType() const {return m_type;}
    wxString getTypeName() const;

    void setFilename(const wxString& newfilename) {m_filename = newfilename;}
    void setExpression(const wxString& newexpression) {m_expression = newexpression;}
    void setClass(const wxString& newclass) {m_class = newclass;}
    void setValue(const wxString& newvalue) {m_value = newvalue;}
    void setFunction(const wxString& newfunction) {m_function = newfunction;}
    void setLine(long newline) {m_line = newline;}
    void setType(Type newtype) {m_type = newtype;}

    StringHash getHash() const;
    
    // operators
    bool operator == (Breakpoint);
        
};

#endif // BREAKPOINT_H
