/*
 * Copyright (C) 2004 by Linus McCabe
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include "PathMapperDialog.h"
#include  <wx/config.h>
#include <wx/listctrl.h>
#include "appGubed.h"

BEGIN_EVENT_TABLE(PathMapperDialog, wxDialog)
EVT_BUTTON(cmd_pathmapperok, PathMapperDialog::OnCommand)
EVT_BUTTON(cmd_pathmappercancel, PathMapperDialog::OnCommand)
EVT_CLOSE(PathMapperDialog::OnClose)
EVT_SIZE(PathMapperDialog::OnSize)
EVT_TEXT(cmd_pathmapperlocalpath, PathMapperDialog::OnPathChange)
EVT_TEXT(cmd_pathmapperserverpath, PathMapperDialog::OnPathChange)
EVT_LIST_ITEM_SELECTED(-1, PathMapperDialog::OnListSelect)
EVT_LIST_ITEM_ACTIVATED(-1, PathMapperDialog::OnListSelect)
EVT_CHOICE(-1, PathMapperDialog::OnChoice)
END_EVENT_TABLE()

PathMapperDialog::PathMapperDialog(wxFrame* parent)
    : wxDialog(parent, -1, _("Map paths"), wxDefaultPosition, wxDefaultSize, wxRESIZE_BORDER | wxDEFAULT_DIALOG_STYLE, _T("PathMapperDialog") )
{

  // Set the icon
  //SetIcon(wxIcon(wxICON(pause)));

  m_convertslashes = 0;

  // Set up controls
  SetupControls();

  // Restore sizes
  wxConfigBase *cfgGubed = wxConfigBase::Get();
  SetSize(cfgGubed->Read(GetName() + wxT("/WindowLeft"), 0l)
        , cfgGubed->Read(GetName() + wxT("/WindowTop"), 0l)
        , cfgGubed->Read(GetName() + wxT("/WindowWidth"), 640l)
        , cfgGubed->Read(GetName() + wxT("/WindowHeight"), 300l));


}

void PathMapperDialog::OnCommand(wxCommandEvent& event)
{
  wxConfigBase *cfgGubed = wxConfigBase::Get();

  switch(event.GetId())
  {
    case cmd_pathmapperok:

      cfgGubed->Write(wxT("PathMapperDialog/ConvertSlashes"), m_choConvertSlashes->GetSelection());
      cfgGubed->Write(wxT("PathMapperDialog/MapHistory_0"), wxString::Format(wxT("%s;%s"), m_LocalPath.c_str(), m_ServerPath.c_str()));

      for(int cnt = 0; cnt < m_historylist->GetItemCount() && cnt < 30; cnt++)
      {
        wxListItem item;
        item.SetId(cnt);
        item.SetColumn(0);
        item.SetMask(wxLIST_MASK_TEXT);
        if((m_historylist->GetItem(item)))
        {
          wxString local = item.GetText();
          item.SetColumn(1);
          if((m_historylist->GetItem(item)))
          {
            wxString server = item.GetText();
            if(local != m_LocalPath && server != m_ServerPath && (local != wxT("") || server != wxT("")))
              cfgGubed->Write(wxString::Format(wxT("PathMapperDialog/MapHistory_%d"), cnt + 1), wxString::Format(wxT("%s;%s"), local.c_str(), server.c_str()));
          }
        }
      }

      cfgGubed->Flush();
      EndModal(wxID_OK);
      break;

    case cmd_pathmappercancel:
      EndModal(wxID_CANCEL);
      break;

  }
}

void PathMapperDialog::OnPathChange(wxCommandEvent &event)
{
  switch(event.GetId())
  {
    case cmd_pathmapperlocalpath:
      if(m_txtLocalPath->GetValue() != m_LocalPath)
      {
        m_LocalPath = m_txtLocalPath->GetValue();
        m_Sample->SetLabel(wxString::Format(_("Sample: %s becomes %s"), m_SamplePath.c_str(), MapLocalToRemote(m_SamplePath).c_str()));
      }
      break;

    case cmd_pathmapperserverpath:
      if(m_txtServerPath->GetValue() != m_ServerPath)
      {
        m_ServerPath = m_txtServerPath->GetValue();
        m_Sample->SetLabel(wxString::Format(_("Sample: %s becomes %s"), m_SamplePath.c_str(), MapLocalToRemote(m_SamplePath).c_str()));
      }
      break;
  }
}

void PathMapperDialog::OnChoice(wxCommandEvent &)
{
  m_convertslashes = m_choConvertSlashes->GetSelection();
  m_Sample->SetLabel(wxString::Format(_("Sample: %s becomes %s"), m_SamplePath.c_str(), MapLocalToRemote(m_SamplePath).c_str()));
}

void PathMapperDialog::OnListSelect(wxListEvent & event)
{
  wxListItem item;
  item.SetId(event.GetIndex());
  item.SetMask(wxLIST_MASK_TEXT);
  item.SetColumn(0);
  if(m_historylist->GetItem(item))
    m_txtLocalPath->SetValue(item.GetText());

  item.SetColumn(1);
  if(m_historylist->GetItem(item))
    m_txtServerPath->SetValue(item.GetText());

  if(event.GetEventType() == wxEVT_COMMAND_LIST_ITEM_ACTIVATED)
    EndModal(wxID_OK);
}

void PathMapperDialog::OnClose(wxCloseEvent& event)
{
  EndModal(wxID_CANCEL);
}

void PathMapperDialog::OnSize(wxSizeEvent& event)
{
  m_historylist->SetColumnWidth(0, event.GetSize().GetWidth() / 2);
  m_historylist->SetColumnWidth(1, event.GetSize().GetWidth() / 2);
  event.Skip(true);
}

wxString PathMapperDialog::MapLocalToRemote(const wxString &localpath)
{
  wxString ret;

  ret = localpath;

  if(localpath.Lower().Left(m_LocalPath.Len()) == m_LocalPath.Lower())
    ret = m_ServerPath + localpath.Mid(m_LocalPath.Len());

  if(m_convertslashes == 1)
    ret.Replace(wxT("/"), wxT("\\"), true);
  else if(m_convertslashes == 2)
    ret.Replace(wxT("\\"), wxT("/"), true);

  return ret;
}

int PathMapperDialog::ShowModal(const wxString & path)
{
  m_SamplePath = path;

  for(int cnt = 0; cnt < m_historylist->GetItemCount(); cnt++)
  {
    if(path.Lower().Left(m_historylist->GetItemText(cnt).Len()) == m_historylist->GetItemText(cnt).Lower())
    {
      wxListItem item;
      item.SetId(cnt);
      item.SetColumn(1);
	  item.SetMask(wxLIST_MASK_TEXT);
      m_historylist->GetItem(item);

      m_txtLocalPath->SetValue(m_historylist->GetItemText(cnt));
      m_txtServerPath->SetValue(item.GetText());
      break;
    }
  }

  m_Sample->SetLabel(wxString::Format(_("Sample: %s becomes %s"), m_SamplePath.c_str(), MapLocalToRemote(m_SamplePath).c_str()));

  return wxDialog::ShowModal();
}

void PathMapperDialog::SetupControls()
{

  wxConfigBase *cfgGubed = wxConfigBase::Get();

  wxBoxSizer * windowsizer = new wxBoxSizer(wxVERTICAL);

  // Labels
  wxBoxSizer * horizsizer = new wxBoxSizer(wxHORIZONTAL);
  horizsizer->Add(new wxStaticText(this, -1, _("Local Path")), 1);
  horizsizer->Add(new wxStaticText(this, -1, _("Server Path")), 1);
  windowsizer->Add(horizsizer, 0, wxEXPAND);

  // Input controls
  horizsizer = new wxBoxSizer(wxHORIZONTAL);
  m_txtLocalPath = new wxTextCtrl(this, cmd_pathmapperlocalpath);
  m_txtServerPath = new wxTextCtrl(this, cmd_pathmapperserverpath);
  horizsizer->Add(m_txtLocalPath, 1);
  horizsizer->Add(m_txtServerPath, 1);

  windowsizer->Add(horizsizer, 0, wxEXPAND);

  // Replace slashes
  wxString strSlashChoices[] = {
                              _("No slash replacement"),
                              _("Replace forward slash (/) with backward slash (\\)"),
                              _("Replace backward slash (\\) with forward slash (/)")};

  horizsizer = new wxBoxSizer(wxHORIZONTAL);
  horizsizer->Add(new wxStaticText(this, -1, _("Replace slashes")), 1, wxALIGN_CENTER_VERTICAL);
  m_choConvertSlashes = new wxChoice(this, -1, wxDefaultPosition, wxDefaultSize, 3, strSlashChoices);
  horizsizer->Add(m_choConvertSlashes, 2, wxEXPAND);
  horizsizer->Add(new wxStaticText(this, -1, wxT("")), 3);
  windowsizer->Add(horizsizer, 0, wxEXPAND);


  // Some space
  windowsizer->Add(10, 20);

  //Sample
  horizsizer = new wxBoxSizer(wxHORIZONTAL);
  m_Sample = new wxStaticText(this, -1, _(""));
  horizsizer->Add(m_Sample, 1);
  windowsizer->Add(horizsizer, 0, wxEXPAND);

  // Some space
  windowsizer->Add(10, 20);

  // History list
  windowsizer->Add(new wxStaticText(this, -1, _("Previous maps")), 0, wxEXPAND);

  m_historylist = new wxListCtrl(this, -1, wxDefaultPosition, wxDefaultSize, wxLC_REPORT | wxLC_SINGLE_SEL | wxLC_HRULES | wxLC_VRULES);
  windowsizer->Add(m_historylist, 1, wxEXPAND);
  m_historylist->InsertColumn(0, _("Local Path"));
  m_historylist->InsertColumn(1, _("Server Path"));


  // Ok and Cancel buttons
  wxBoxSizer * buttonsizer = new wxBoxSizer(wxHORIZONTAL);
  buttonsizer->Add(1, 1, 1, wxEXPAND);
  buttonsizer->Add(new wxButton(this, cmd_pathmapperok, _("OK"), wxDefaultPosition, wxDefaultSize), 0, wxALL, 4);
  buttonsizer->Add(new wxButton(this, cmd_pathmappercancel, _("Cancel"), wxDefaultPosition, wxDefaultSize), 0, wxALL, 4);
  windowsizer->Add(buttonsizer, 0, wxEXPAND);

  // Alter controls from settings
  int cnt = 0;
  while(true)
  {
    wxString pair;
    pair = cfgGubed->Read(wxString::Format(wxT("PathMapperDialog/MapHistory_%d"), cnt), wxT(""));
    if(pair == "")
      break;

    wxString local = pair.BeforeFirst(';');
    wxString server = pair.AfterFirst(';');

    m_historylist->InsertItem(cnt, local);
    m_historylist->SetItem(cnt, 1, server);

    cnt++;
  }

  m_convertslashes = cfgGubed->Read(wxT("PathMapperDialog/ConvertSlashes"), 0l);
  m_choConvertSlashes->SetSelection(m_convertslashes);

  SetSizer(windowsizer);
//  windowsizer->SetSizeHints(this);



}

PathMapperDialog::~PathMapperDialog()
{
  wxConfigBase *cfgGubed = wxConfigBase::Get();

  wxRect rctPos = this->GetRect();
  cfgGubed->Write(GetName() + wxT("/WindowLeft"), rctPos.GetLeft());
  cfgGubed->Write(GetName() + wxT("/WindowTop"), rctPos.GetTop());
  cfgGubed->Write(GetName() + wxT("/WindowWidth"), rctPos.GetWidth());
  cfgGubed->Write(GetName() + wxT("/WindowHeight"), rctPos.GetHeight());
  cfgGubed->Flush();

}
