/*
 * Copyright (C) 2004 by Linus McCabe
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "appGubed.h"
#include "wxClientGlobal.h"
#include "Breakpoint.h"
#include <wx/config.h>
#include <wx/image.h>
#include <wx/filesys.h>
#include <wx/fs_zip.h>

IMPLEMENT_APP(appGubed);


bool appGubed::OnInit()
{
  int intTop, intLeft, intWidth, intHeight;

  wxLog::SetLogLevel(wxLOG_Info);
  
  ::wxInitAllImageHandlers();
  wxFileSystem::AddHandler(new wxZipFSHandler); 
  m_displaydelay = 15;
  m_currentbm = 0;
  m_currentbp = 0;
  m_connectionhandler = NULL;

  SetVendorName("JLDesign");
  SetAppName("Gubed");

  wxConfigBase *cfgGubed = wxConfigBase::Get();

  intLeft = cfgGubed->Read("WindowLeft", 0l);
  intTop = cfgGubed->Read("WindowTop", 0l);
  intWidth = cfgGubed->Read("WindowWidth", 640l);
  intHeight = cfgGubed->Read("WindowHeight", 480l);

  wxPoint posPos(intLeft, intTop);
  wxSize sizSize(intWidth, intHeight);

  frmGubed *frmMain = new frmGubed( PROGNAMEVER, posPos, sizSize );
  SetTopWindow(frmMain);
  frmMain->Show(TRUE);

  setExecutionState(ExecutionStates::Pause);
  DebugMode(false);
  m_connectionhandler = new wxSocketLineHandler();
  frmMain->setConnectionState();

  return TRUE;
}

int appGubed::OnExit()
{
  if(m_connectionhandler)
  {
    // Unless we're using the proxy, make sure the client disconnectd before we delete the connectiohandler
    wxConfigBase *cfgGubed = wxConfigBase::Get();
    if(!cfgGubed->Read(wxT("UseProxy"), 0l))
    {
      int tries = 0;

      // Try for 10 times or until we lose connection
      while(isConnected() && tries < 10 )
      {
        SendCommand(wxT("die"));
        ::wxYield();
        ::wxSleep(1);
        tries++;
      }
      if(isConnected())
        wxLogWarning(_T("Script refused to disconnect!"));
    }
    delete m_connectionhandler;
    m_connectionhandler = NULL;
  }
  return 0;
}


void appGubed::Reconfigure()
{
  if(m_connectionhandler && !m_connectionhandler->isConnected())
  {
    delete m_connectionhandler;
    m_connectionhandler = new wxSocketLineHandler();
  }
}

void appGubed::Connect()
{
 if(m_connectionhandler)
    m_connectionhandler->Connect();
}

bool appGubed::isConnected()
{
  if(m_connectionhandler)
    return m_connectionhandler->isConnected();

  return false;
}

bool appGubed::isListening()
{
  if(m_connectionhandler)
    return m_connectionhandler->isListening();

  return false;
}

void appGubed::setConnectionState()
{
  if(GetTopWindow())
    GetGubedWindow()->setConnectionState();

}
void appGubed::DebugMode(bool mode)
{
  if(GetTopWindow())
  {
    GetGubedWindow()->DebugMode(mode);

    if(!mode)
      GetGubedWindow()->GetSources()->SetActiveSourceLine(_T(""), 0);
  }
}

void appGubed::SendCommand(const wxString& strCommand)
{
  if(m_connectionhandler)
    m_connectionhandler->SendCommand(strCommand, (char*)0);

}

void appGubed::SendCommand(const wxString& strCommand, StringHash args)
{
  if(m_connectionhandler)
    m_connectionhandler->SendCommand(strCommand, args);
}

frmGubed * appGubed::GetGubedWindow(void)
{
  return (frmGubed * )GetTopWindow();
}
void appGubed::setExecutionState(ExecutionStates::State newstate)
{
  if(newstate == ExecutionStates::Pause)
  {

    wxGetApp().SendCommand(wxT("pause"));
    wxGetApp().SendCommand(wxT("sendactiveline"));
  }
  else if(newstate == ExecutionStates::Run)
  {
    if(m_executionstate == ExecutionStates::Pause)
      wxGetApp().SendCommand(wxT("next"));

    wxGetApp().SendCommand(wxT("run"));

  }
  else if(newstate == ExecutionStates::Trace)
  {
    if(m_executionstate == ExecutionStates::Pause)
      wxGetApp().SendCommand(wxT("next"));

    wxGetApp().SendCommand(wxT("trace"));
  }

  m_executionstate = newstate;
  GetGubedWindow()->ShowState(m_executionstate);
}
/*
void appGubed::ConditionalBreakpoint(const wxString &expression, const wxString &filename, const wxString &cclass, const wxString &function, const wxString &type)
{
  wxString condition;
  bool hasbp = false;

  condition = wxT("^") + filename + wxT("^") + cclass + wxT("^") + function + wxT("^") + type + wxT('^') + expression;
  for(unsigned int idx = 0; idx < m_conditionalbreakpointlist.Count(); idx++) {
    if(m_conditionalbreakpointlist[idx] == condition)
    {
      hasbp = true;
      break;
    }
  }
  if(!hasbp)
  {
    m_conditionalbreakpointlist.Add(condition);
    // If we're connected, tell the script and it will tell us back.. otherwise just add the bp
    if(isConnected())
      wxGetApp().SendCommand(wxT("conditionalbreakpoint"), CompileHash("condition", condition.c_str(), 0));
    else
      GetGubedWindow()->ShowCondition(condition + wxT("^N"));
      
  }
  return;
}

bool appGubed::clearConditionalBreakpoint(const wxString &bp)
{
  bool hasbp = false;
  for(unsigned int idx = 0; idx < m_conditionalbreakpointlist.Count(); idx++) {
    wxLogDebug(wxT("Comparing '%s' and '%s'"), m_conditionalbreakpointlist[idx].c_str(), bp.c_str());
    if(m_conditionalbreakpointlist[idx] == bp)
    {
      hasbp = true;
      break;
    }
  }
  if(hasbp)
  {
    m_conditionalbreakpointlist.Remove(bp);
    wxGetApp().SendCommand(wxT("clearconditionalbreakpoint"), CompileHash("breakpoint", bp.c_str(), 0));
    return true;
  }
  wxLogWarning(_("Unable to remove conditional breakpoint \"%s\" "), bp.c_str());
  return false;
}
*/
void appGubed::setBreakpoint(const Breakpoint& bp)
{
  bool hasbp = false;
  for(unsigned int idx = 0; idx < m_breakpointlist.Count(); idx++) {
    if(m_breakpointlist[idx] == bp)
    {
      hasbp = true;
      break;
    }
  }
  if(!hasbp)
  {
    m_breakpointlist.Add(bp);
    wxGetApp().SendCommand(wxT("breakpoint"), bp.getHash());
    GetGubedWindow()->updateBreakpoint((int)m_breakpointlist.Count() - 1, bp);
  }
}

void appGubed::updateBreakpoint(const Breakpoint& bp)
{
  for(unsigned int idx = 0; idx < m_breakpointlist.Count(); idx++) {
    if(m_breakpointlist[idx] == bp)
    {
      GetGubedWindow()->updateBreakpoint((int)idx, bp);
      return;
    }
  }
}

void appGubed::clearBreakpoint(const Breakpoint& bp)
{
  for(unsigned int idx = 0; idx < m_breakpointlist.Count(); idx++) {
    if(m_breakpointlist[idx] == bp)
    {
      m_breakpointlist.RemoveAt(idx);
      SendCommand(wxT("removebreakpoint"), bp.getHash());
      GetGubedWindow()->removeBreakpoint(idx);
      break;
    }
  }
}

void appGubed::clearBreakpoint(int idx)
{
  if(idx < (int)m_breakpointlist.Count())
  {
    SendCommand(wxT("removebreakpoint"), m_breakpointlist[idx].getHash());
    m_breakpointlist.RemoveAt(idx);
    GetGubedWindow()->removeBreakpoint(idx);
  }
}

void appGubed::sendBreakpoints()
{
  for(unsigned int idx = 0; idx < m_breakpointlist.Count(); idx++) {
    SendCommand(wxT("breakpoint"), m_breakpointlist[idx].getHash()); 
  }
}


void appGubed::setBookmark(const wxString &source, long line)
{
  wxString data;
  data << source + wxT(":") << line;

  bool hasbp = false;
  for(unsigned int idx = 0; idx < m_bookmarklist.Count(); idx++) {
    if(m_bookmarklist[idx] == data)
    {
      hasbp = true;
      break;
    }
  }
  if(!hasbp)
    m_bookmarklist.Add(data);
}

void appGubed::clearBookmark(const wxString &source, long line)
{
  wxString data;
  data << source + wxT(":") << line;

  bool hasbp = false;
  for(unsigned int idx = 0; idx < m_bookmarklist.Count(); idx++) {
    if(m_bookmarklist[idx] == data)
    {
      hasbp = true;
      break;
    }
  }
  if(hasbp)
    m_bookmarklist.Remove(data);
}

void appGubed::FindBookmark(int dir)
{
  if(m_bookmarklist.Count() <= 0)
    return;

  m_currentbm += dir;
  if(m_currentbm < 0)
    m_currentbm = m_bookmarklist.Count() - 1;

  if(m_currentbm >= m_bookmarklist.Count())
    m_currentbm = 0;

  long line;
  m_bookmarklist[m_currentbm].AfterFirst(':').ToLong(&line);
  wxGetApp().GetGubedWindow()->GetSources()->gotoFileAndLine(m_bookmarklist[m_currentbm].BeforeFirst(':'), line);

}

void appGubed::FindBreakpoint(int dir)
{
  if(m_breakpointlist.Count() <= 0)
    return;

  unsigned long oldcurrent = m_currentbp;
    
  do {
    m_currentbp += dir;
    if(m_currentbp < 0)
      m_currentbp = m_breakpointlist.Count() - 1;
  
    if(m_currentbp >= m_breakpointlist.Count())
      m_currentbp = 0;
  } while(m_breakpointlist[m_currentbp].getType() != Breakpoint::Line && oldcurrent != m_currentbp);
  
  wxGetApp().GetGubedWindow()->GetSources()->gotoFileAndLine(m_breakpointlist[m_currentbp].getFilename(), m_breakpointlist[m_currentbp].getLine());

}

wxArrayInt appGubed::getFileBreakpoints(const wxString & filename)
{
  wxArrayInt filebps;

  for(unsigned int bp = 0; bp < m_breakpointlist.Count(); bp++)
  {
    if(m_breakpointlist[bp].getFilename() == filename)
      filebps.Add(m_breakpointlist[bp].getLine());
  }

  return filebps;
}



void appGubed::setDisplayDelay(long newdelay)
{

  if(newdelay >= 0)
    m_displaydelay = newdelay;

  wxString delay;
  delay << m_displaydelay;
  wxGetApp().SendCommand(wxT("setdisplaydelay"), CompileHash("newdelay", delay.c_str(), 0));
}

void appGubed::AddWatch(wxString watch)
{
  bool watching = false;
  for(unsigned int idx = 0; idx < m_watchlist.Count() && watching == false; idx++)
  {
    if(m_watchlist[idx] == watch)
      watching = true;
  }
  if(!watching)
  {
    m_watchlist.Add(watch);
    if(isConnected())
      SendCommand(wxT("getwatch"), CompileHash("variable", watch.c_str(), 0));
    else
    {
      wxString see;// NUll, Example: s:6:"return";N;
      see << wxT("s:") << (watch.Length() + 1) << wxT(":$") << watch << wxT(":N;");
      GetGubedWindow()->ShowWatch(see);
    }
  }
}
void appGubed::RemoveWatch(wxString watch)
{
  bool watching = false;
  for(unsigned int idx = 0; idx < m_watchlist.Count() && watching == false; idx++)
  {
    if(m_watchlist[idx] == watch)
      watching = true;
  }
  if(watching)
  {
    m_watchlist.Remove(watch);
  }
}

void appGubed::SendWatches() {
  static int watchkey = 0;
  watchkey++;

  if(wxGetApp().m_watchlist.Count() > 0) {

    // Tell client to expect variables
    GetGubedWindow()->PrepareWatches(watchkey, false);

    // Request variables from program
    for(unsigned int idx = 0; idx < m_watchlist.Count(); idx++) {
        SendCommand(wxT("getwatch"), CompileHash("variable", m_watchlist[idx].c_str(), 0));
    }

  }

  // Tell program to send clearance when done
  // We do this wether we have watches or not (so old views are cleared)
  wxString key;
  key << watchkey;
  SendCommand(wxT("sentwatches"), CompileHash("key", key.c_str(), 0));

}
