/*
 * Copyright (C) 2004 by Linus McCabe
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include "frmConditionalBreakpoint.h"
#include  <wx/config.h>
#include "appGubed.h"
#include "Breakpoint.h"

BEGIN_EVENT_TABLE(frmConditionalBreakpoint, wxDialog)
EVT_BUTTON(wxID_OK, frmConditionalBreakpoint::OnCommand)
EVT_BUTTON(wxID_CANCEL, frmConditionalBreakpoint::OnCommand)
// EVT_CHECKBOX(cmd_settingsproxy, frmConditionalBreakpoint::OnCommand)
EVT_CLOSE(frmConditionalBreakpoint::OnClose)
END_EVENT_TABLE()

frmConditionalBreakpoint::frmConditionalBreakpoint(wxFrame* parent)
    : wxDialog(parent, -1, _("Conditional breakpoint"), wxDefaultPosition, wxDefaultSize, wxRESIZE_BORDER | wxDEFAULT_DIALOG_STYLE, _T("ConditionalBreakpointDialog") )
{

  // Set the icon
  //SetIcon(wxIcon(wxICON(pause)));

  // Set up controls
  SetupControls();

  // Restore sizes
  wxConfigBase *cfgGubed = wxConfigBase::Get();
  SetSize(cfgGubed->Read(GetName() + wxT("/WindowLeft"), 0l)
        , cfgGubed->Read(GetName() + wxT("/WindowTop"), 0l)
        , cfgGubed->Read(GetName() + wxT("/WindowWidth"), 640l)
        , cfgGubed->Read(GetName() + wxT("/WindowHeight"), 300l));

}

wxString frmConditionalBreakpoint::GetExpression()
{
  return m_expression->GetValue();
}
wxString frmConditionalBreakpoint::GetFunction()
{
  return m_function->GetValue();
}
wxString frmConditionalBreakpoint::GetClass()
{
  return m_class->GetValue();
}
wxString frmConditionalBreakpoint::GetFilename()
{
  return m_filename->GetValue();
}
Breakpoint::Type frmConditionalBreakpoint::GetType()
{
  return m_breakontrue->GetValue() ? Breakpoint::True : Breakpoint::Change;
}

void frmConditionalBreakpoint::SetExpression(const wxString &expression)
{
  m_expression->SetValue(expression);
}

void frmConditionalBreakpoint::SetClass(const wxString &cclass)
{
  m_class->SetValue(cclass);
}

void frmConditionalBreakpoint::SetFunction(const wxString &func)
{
  m_function->SetValue(func);
}

void frmConditionalBreakpoint::SetFilename(const wxString &filename)
{
  m_filename->SetValue(filename);
}


void frmConditionalBreakpoint::OnCommand(wxCommandEvent& event)
{
  EndModal(event.GetId());
//   switch(event.GetId())
//   {
//     case wxID_OK:
//       EndModal(wxID_OK);
//       break;
//
//     case wxID_CANCEL:
//       EndModal(wxID_CANCEL);
//       break;
//
//   }

}

void frmConditionalBreakpoint::OnClose(wxCloseEvent& event)
{
  EndModal(wxID_CANCEL);
}

void frmConditionalBreakpoint::SetupControls()
{
 // wxBoxSizer * topsizer = new wxBoxSizer(wxVERTICAL);
  wxFlexGridSizer * windowsizer = new wxFlexGridSizer(2);//wxVERTICAL);
  windowsizer->AddGrowableCol(1);

  // Label
  wxStaticText *stLabel = new wxStaticText(this, -1, _("Add conditional breakpoint"), wxDefaultPosition, wxDefaultSize);
  windowsizer->Add(stLabel, 0, wxEXPAND | wxALL, 4);
  windowsizer->Add(1, 1, 1, wxEXPAND);

  stLabel = new wxStaticText(this, -1, _("Expression"), wxDefaultPosition, wxDefaultSize);
  m_expression = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize);
  windowsizer->Add(stLabel, 0, wxEXPAND | wxALL, 4);
  windowsizer->Add(m_expression, 0, wxEXPAND | wxWEST | wxEAST, 4);

  stLabel = new wxStaticText(this, -1, _("Only break in file"), wxDefaultPosition, wxDefaultSize);
  m_filename = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize);
  windowsizer->Add(stLabel, 0, wxEXPAND | wxALL, 4);
  windowsizer->Add(m_filename, 0, wxEXPAND | wxWEST | wxEAST, 4);

  stLabel = new wxStaticText(this, -1, _("Only break in objects of class"), wxDefaultPosition, wxDefaultSize);
  m_class = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize);
  windowsizer->Add(stLabel, 0, wxEXPAND | wxALL, 4);
  windowsizer->Add(m_class, 0, wxEXPAND | wxWEST | wxEAST, 4);

  stLabel = new wxStaticText(this, -1, _("Only break in function"), wxDefaultPosition, wxDefaultSize);
  m_function = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize);
  windowsizer->Add(stLabel, 0, wxEXPAND | wxALL, 4);
  windowsizer->Add(m_function, 0, wxEXPAND | wxWEST | wxEAST, 4);

  stLabel = new wxStaticText(this, -1, _("Break when"), wxDefaultPosition, wxDefaultSize);
  windowsizer->Add(stLabel, 0, wxEXPAND | wxALL, 4);
  m_breakontrue = new wxRadioButton(this, -1, _("expression is true"));
  windowsizer->Add(m_breakontrue, 0, wxEXPAND | wxWEST | wxEAST, 4);
  windowsizer->Add(1, 1, 1, wxEXPAND);
  m_breakonchange = new wxRadioButton(this, -1, _("expression changes"));
  windowsizer->Add(m_breakonchange, 0, wxEXPAND | wxWEST | wxEAST, 4);

  wxBoxSizer * buttonsizer = new wxBoxSizer(wxHORIZONTAL);
  buttonsizer->Add(1, 1, 1, wxEXPAND);
  buttonsizer->Add(new wxButton(this, wxID_OK, _("OK"), wxDefaultPosition, wxDefaultSize), 0, wxALL, 4);
  buttonsizer->Add(new wxButton(this, wxID_CANCEL, _("Cancel"), wxDefaultPosition, wxDefaultSize), 0, wxALL, 4);
  windowsizer->Add(1, 1, 1, wxEXPAND);
  windowsizer->Add(buttonsizer, 0, wxEXPAND);

  //topsizer->Add(windowsizer, 1, wxEXPAND);
  SetSizer(windowsizer);
  windowsizer->SetSizeHints(this);


}

frmConditionalBreakpoint::~frmConditionalBreakpoint()
{
  wxConfigBase *cfgGubed = wxConfigBase::Get();

  wxRect rctPos = this->GetRect();
  cfgGubed->Write(GetName() + wxT("/WindowLeft"), rctPos.GetLeft());
  cfgGubed->Write(GetName() + wxT("/WindowTop"), rctPos.GetTop());
  cfgGubed->Write(GetName() + wxT("/WindowWidth"), rctPos.GetWidth());
  cfgGubed->Write(GetName() + wxT("/WindowHeight"), rctPos.GetHeight());
  cfgGubed->Flush();

}
