/*
 * Copyright (C) 2004 by Linus McCabe
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include "appGubed.h"
#include "frmGubed.h"
#include "frmSettings.h"
#include "PathMapperDialog.h"
#include "frmConditionalBreakpoint.h"
#include <wx/config.h>
#include <wx/menuitem.h>
#include "tidWatchData.h"
#include <wx/tipwin.h>
#include <wx/textfile.h>
#include <wx/notebook.h>
#include <wx/grid.h>
#include <wx/colour.h>
#include <wx/utils.h>
#include <wx/button.h>

#ifdef __WIN32__
	#include "../configmingw.h"
#else
	#include "../config.h"
#endif

#define USE_XPM_BITMAPS 1

#include "bitmaps/closefile.xpm"
#include "bitmaps/openfile.xpm"
#include "bitmaps/pause.xpm"
#include "bitmaps/trace.xpm"
#include "bitmaps/run.xpm"
#include "bitmaps/next.xpm"
#include "bitmaps/prev.xpm"
#ifndef __WIN32__
	#include "bitmaps/gubed.xpm"
#endif
#include "bitmaps/stepinto.xpm"
#include "bitmaps/stepout.xpm"
#include "bitmaps/stepover.xpm"
#include "bitmaps/skip.xpm"
#include "bitmaps/kill.xpm"
#include "bitmaps/bookmark.xpm"
#include "bitmaps/breakpoint.xpm"
#include "bitmaps/quit.xpm"
#include "bitmaps/connect.xpm"
#include "bitmaps/disconnect.xpm"
#include "bitmaps/currentline.xpm"

BEGIN_EVENT_TABLE(frmGubed, wxFrame)
  EVT_MENU(cmd_quit, frmGubed::OnQuit)
  EVT_MENU(cmd_about, frmGubed::OnAbout)
  EVT_MENU(cmd_help, frmGubed::OnHelp)
  EVT_MENU(cmd_settings, frmGubed::OnSettings)
  EVT_MENU(cmd_openfile, frmGubed::OnOpenFile)
  EVT_MENU(cmd_closefile, frmGubed::OnCloseFile)

  EVT_BUTTON(cmd_about, frmGubed::OnAbout)
  EVT_BUTTON(cmd_help, frmGubed::OnHelp)

  EVT_TOOL(cmd_connect, frmGubed::OnConnect)
  EVT_TOOL(cmd_disconnect, frmGubed::OnConnect)
  EVT_TOOL_RANGE(cmd_controlsfirst, cmd_controlslast, frmGubed::OnCommand)

  EVT_TOOL_RANGE(cmd_bookmarkFirst, cmd_bookmarkLast, frmGubed::OnBookmark)
  EVT_TOOL_RANGE(cmd_breakpointFirst, cmd_breakpointLast, frmGubed::OnBreakpoint)

  EVT_TOOL_RANGE(cmd_logFirst, cmd_logLast, frmGubed::OnLog)
  EVT_TOOL_RANGE(cmd_variablesFirst, cmd_variablesLast, frmGubed::OnVariables)
  EVT_TOOL_RANGE(cmd_watchFirst, cmd_watchLast, frmGubed::OnWatch)

  EVT_TREE_KEY_DOWN(-1, frmGubed::OnWatchKeyDown)
  
  EVT_GRID_CELL_LEFT_DCLICK(frmGubed::OnGridDbl)
  EVT_GRID_CELL_RIGHT_CLICK(frmGubed::OnGridRightClick)
  EVT_SIZE(frmGubed::OnReSize)
  
  EVT_BUTTON(cmd_command, frmGubed::OnLog)
  
  EVT_COMMAND_SCROLL/*_THUMBRELEASE*/(cmd_tracedelay, frmGubed::OnScroll)
  /*EVT_COMMAND_SCROLL_LINEUP(cmd_tracedelay, frmGubed::OnScroll)
  EVT_COMMAND_SCROLL_LINEDOWN(cmd_tracedelay, frmGubed::OnScroll)
  EVT_COMMAND_SCROLL_PAGEUP(cmd_tracedelay, frmGubed::OnScroll)
  EVT_COMMAND_SCROLL_PAGEDOWN(cmd_tracedelay, frmGubed::OnScroll)*/


END_EVENT_TABLE()

frmGubed::frmGubed(const wxString& title, const wxPoint& pos, const wxSize& size)
    : wxFrame((wxFrame *)NULL, -1, title, pos, size)
{

  // Clear some vars
  m_tbMainToolbar = NULL;
  m_nbSourceLists = NULL;
  m_txtLog = NULL;

  m_swHorizontal = NULL;
  m_swVertical = NULL;

  m_tipwindow = NULL;
  m_tcWatches = NULL;

  m_backtracelist = NULL;
  m_breakpointlist = NULL;
  
  // First of all and most importantly, set the icon
  SetIcon(wxIcon(wxICON(gubed)));

  // Set up less important junk like menues, toolbar and controls
  SetupMenues();
  SetupToolbar();
  SetupControls();

  //CreateStatusBar();
  ::wxLogMessage( _("Welcome to " PROGNAME ));

  m_viewroot = ReplaceWatch(_("Shown variables"), "a", 0, -1 , _(""), m_tcWatches->GetRootItem() );
  m_tcWatches->SetItemBold(m_viewroot);
  m_watchroot = ReplaceWatch(_("Watches"), "a", 0, -1 , _(""), m_tcWatches->GetRootItem() );
  m_tcWatches->SetItemBold(m_watchroot);
  
  m_help.UseConfig(wxConfig::Get());
  m_help.SetTempDir(wxT("."));
  
  wxString helpfile;
  if(wxFileExists(wxT("gubedhelp.zip")))
    helpfile = wxT("gubedhelp.zip");
  else if(wxFileExists(wxT(TOP_SRCDIR "gubedhelp.zip"))) 
    helpfile = wxT(TOP_SRCDIR "gubedhelp.zip");
  else if(wxFileExists(wxT(DATADIR "gubedhelp.zip"))) 
    helpfile = wxT(DATADIR "gubedhelp.zip");
  if(helpfile.IsEmpty())
    wxLogWarning(_T("Unable to find help file (searched in current dir and %s for %s)"), DATADIR, "gubedhelp.zip");
  else
    if(!m_help.AddBook(wxFileName(helpfile, wxPATH_UNIX)))
      wxLogWarning(wxT("Failed adding the helpfile (%s)"), helpfile.c_str());

}

void frmGubed::OnQuit(wxCommandEvent& WXUNUSED(event))
{
  Close(TRUE);
}


void frmGubed::OnLogEnter(wxCommandEvent& event)
{
/*  switch(event.GetId())
  {
    case cmd_logcommand:
      if(wxGetKeyState(WXK_CONTROL)) //wx 2.5...
      {
      if(wxGetApp().isConnected())
      {
        m_txtLog->AppendText(m_txtCommand->GetValue() + _T("\n"));
        wxGetApp().SendCommand(_T("processphp"), CompileHash("command", m_txtCommand->GetValue().c_str(), 0));
        m_txtCommand->SetValue(_T(""));
      }
      break;
      
    default:
      event.Skip();
  }*/
}

void frmGubed::OnWatchKeyDown(wxTreeEvent& event)
{

  if( event.GetKeyCode() == WXK_DELETE)
  {
    if(m_tcWatches->GetSelection())
    {
      tidWatchData *tidWatch;
      tidWatch = (tidWatchData *) m_tcWatches->GetItemData(m_tcWatches->GetSelection());

      if(m_tcWatches->GetItemParent(m_tcWatches->GetSelection()) == m_watchroot)
      {
        wxGetApp().RemoveWatch(tidWatch->strVariable);
        m_hmwWatches.erase(tidWatch->strWatch);
        m_tcWatches->Delete(m_tcWatches->GetSelection());
        return;
      }
    }
  }
  event.Skip();
}

void frmGubed::OnScroll(wxScrollEvent& event)
{
  wxGetApp().setDisplayDelay(event.GetInt());
}

void frmGubed::OnCommand(wxCommandEvent& event)
{

  switch(event.GetId())
  {
    case cmd_stepinto:
      wxGetApp().SendCommand(wxT("next"));
      break;

    case cmd_stepover:
      wxGetApp().SendCommand(wxT("stepover"));
      break;

    case cmd_stepout:
      wxGetApp().SendCommand(wxT("stepout"));
      break;

    case cmd_skip:
      wxGetApp().SendCommand(wxT("skip"));
      break;

    case cmd_trace:
      wxGetApp().setExecutionState(ExecutionStates::Trace);
      break;

    case cmd_run:
      wxGetApp().setExecutionState(ExecutionStates::Run);
      break;

    case cmd_pause:
      wxGetApp().setExecutionState(ExecutionStates::Pause);
      break;

    case cmd_kill:
      wxGetApp().SendCommand(wxT("die"));
      break;

    case cmd_findsourceline:
      m_nbSourceLists->ShowActiveSourceLine();
      break;
  }

}

void frmGubed::OnSettings(wxCommandEvent& WXUNUSED(event))
{
  frmSettings * frmSet;
  frmSet = new frmSettings(_("Settings"), wxDefaultPosition, wxDefaultSize);
  frmSet->ShowModal();
  frmSet->Destroy();

  SetupToolbar();
  Layout();
  Refresh();
  wxGetApp().Reconfigure();
  setConnectionState();
}

void frmGubed::OnAbout(wxCommandEvent& WXUNUSED(event))
{

  wxMessageBox(PROGNAMEVER,
               PROGNAMEVER, wxOK | wxICON_INFORMATION, this);

}

void frmGubed::OnHelp(wxCommandEvent& WXUNUSED(event))
{
  m_help.DisplayContents();
}


void frmGubed::OnGridDbl(wxGridEvent &evt)
{
  switch(evt.GetId())
  {
    case grd_backtracelist:
      GetSources()->gotoFileAndLine(m_backtracelist->GetCellValue(evt.GetRow(), BacktraceColumns::Filename).Trim(), wxAtoi(m_backtracelist->GetCellValue(evt.GetRow(), BacktraceColumns::Line)) - 1);
    break;
    
    case grd_breakpointlist:
      if(!m_breakpointlist->GetCellValue(evt.GetRow(), BreakpointColumns::Filename).Trim().IsEmpty())
      {
        GetSources()->gotoFileAndLine(m_breakpointlist->GetCellValue(evt.GetRow(), BreakpointColumns::Filename).Trim(), wxAtoi(m_breakpointlist->GetCellValue(evt.GetRow(), BreakpointColumns::Line)) - 1);
      }
    break;
  }
}

void frmGubed::OnGridRightClick(wxGridEvent& event)
{
  switch(event.GetId())
  {
    case grd_breakpointlist:
      m_breakpointlist->SelectRow(event.GetRow());
      wxMenu menu(_("Breakpoint"));
      menu.Append(cmd_breakpoint_remove, _("Remove"));

      PopupMenu(&menu, event.GetPosition().x, m_breakpointlist->ClientToScreen(event.GetPosition()).y - ClientToScreen(GetPosition()).y - event.GetPosition().y);
      return;
  }
  event.Skip();
  return;
}

void frmGubed::OnBreakpoint(wxCommandEvent& event)
{
  frmConditionalBreakpoint *condition;

  switch(event.GetId())
  {
    case cmd_conditionalbreakpoint:
      condition =
        new frmConditionalBreakpoint(this);

      if(condition->ShowModal() ==  wxID_OK)
      {
        if(condition->GetExpression() != _T(""))
        {
          Breakpoint bp(condition->GetType(), 
                        condition->GetExpression(),
                        condition->GetFilename(),
                        condition->GetClass(),
                        condition->GetFunction()
                        );
          wxGetApp().setBreakpoint(bp);
        }
      }
      delete condition;
      break;
    case cmd_breakpoint_remove:
      if(m_breakpointlist->GetSelectedRows().Count() > 0)
      {
        int idx = m_breakpointlist->GetSelectedRows()[0];
        m_nbSourceLists->ClearBreakpoint(m_breakpointlist->GetCellValue(idx, BreakpointColumns::Filename).Trim(), wxAtoi(m_breakpointlist->GetCellValue(idx, BreakpointColumns::Line)) - 1);
        wxGetApp().clearBreakpoint(idx);
      }
      break;    
    case cmd_breakpoint:
      m_nbSourceLists->ToggleBreakpoint(-1);
      break;
    case cmd_nextbreakpoint:
      wxGetApp().FindBreakpoint(1);
      break;
    case cmd_prevbreakpoint:
      wxGetApp().FindBreakpoint(-1);
      break;
  }
}

void frmGubed::OnBookmark(wxCommandEvent& event)
{
  switch(event.GetId())
  {
    case cmd_bookmark:
      m_nbSourceLists->ToggleBookmark(-1);
      break;
    case cmd_nextbookmark:
      wxGetApp().FindBookmark(1);
      break;
    case cmd_prevbookmark:
      wxGetApp().FindBookmark(-1);
      break;
  }
}

void frmGubed::OnConnect(wxCommandEvent& WXUNUSED(event)) {
  wxGetApp().Connect();
}

void frmGubed::setConnectionState()
{
  m_tbMainToolbar->EnableTool(cmd_connect, !wxGetApp().isListening() && !wxGetApp().isConnected());
  m_tbMainToolbar->EnableTool(cmd_disconnect, wxGetApp().isConnected() && !wxGetApp().isListening());
  GetMenuBar()->Enable(cmd_connect,  !wxGetApp().isListening() && !wxGetApp().isConnected());
  GetMenuBar()->Enable(cmd_disconnect, wxGetApp().isConnected() && !wxGetApp().isListening());
  
  if(wxGetApp().isConnected())
    m_txtCommand->SetBackgroundColour(wxColour(255, 255, 255));
  else
    m_txtCommand->SetBackgroundColour(wxColour(164, 164, 164));
  m_txtCommand->Refresh();
  m_btnCommand->Enable(wxGetApp().isConnected());
}

void frmGubed::OnLog(wxCommandEvent& event)
{

  switch(event.GetId())
  {
    case cmd_logsave:
    {
      wxConfigBase *cfgGubed = wxConfigBase::Get();

      wxFileDialog dialog(this, _("Save log as"), cfgGubed->Read(wxT("GUI/LogDirectory")), _("GubedLog.txt"), wxT("*.*"), wxSAVE | wxOVERWRITE_PROMPT);
      if(dialog.ShowModal())
      {
        m_txtLog->SaveFile(dialog.GetPath());

        cfgGubed->Write(wxT("GUI/LogDirectory"), dialog.GetDirectory());
        cfgGubed->Flush();
      }
    }
    break;

    case cmd_logclear:
    {
      m_txtLog->Clear();
    }
    break;

    case cmd_logverbose:
    {
      wxMenuItem* menu = GetMenuBar()->FindItem(cmd_logverbose);
      if(menu)
        wxLog::SetVerbose(menu->IsChecked());
        wxLog::SetLogLevel(wxLOG_Info);
    }
    break;
    
    case cmd_logdebug:
    {
      wxMenuItem* menu = GetMenuBar()->FindItem(cmd_logdebug);
      if(menu)
        if(menu->IsChecked())
          wxLog::SetLogLevel(wxLOG_Debug);
        else
          wxLog::SetLogLevel(wxLOG_Info);
    }
    break;
    
    case cmd_command:
    {
      m_txtLog->AppendText(m_txtCommand->GetValue() + _T("\n"));
      wxGetApp().SendCommand(_T("processphp"), CompileHash("command", m_txtCommand->GetValue().c_str(), 0));
      m_txtCommand->SetValue(_T(""));
    }
    break;
    
  }

}

void frmGubed::OnVariables(wxCommandEvent& event)
{
  wxTextEntryDialog *teValue;

  switch(event.GetId())
  {
    case cmd_variablesgetall:
       wxGetApp().SendCommand(wxT("sendallvariables"));
      break;
    case cmd_variablesgetserver:
       wxGetApp().SendCommand(wxT("getvariable"), CompileHash("variable", wxT("$_SERVER"), 0));
      break;
    case cmd_variablesgetsession:
       wxGetApp().SendCommand(wxT("getvariable"), CompileHash("variable", wxT("$_SESSION"), 0));
      break;
    case cmd_variablesgetenv:
       wxGetApp().SendCommand(wxT("getvariable"), CompileHash("variable", wxT("$_ENV"), 0));
      break;
    case cmd_variablesgetthis:
       wxGetApp().SendCommand(wxT("getvariable"), CompileHash("variable", wxT("$this"), 0));
      break;
    case cmd_variablesgetglobals:
       wxGetApp().SendCommand(wxT("getvariable"), CompileHash("variable", wxT("$GLOBALS"), 0));
      break;
    case cmd_variablesgetget:
       wxGetApp().SendCommand(wxT("getvariable"), CompileHash("variable", wxT("$_GET"), 0));
      break;
    case cmd_variablesgetrequest:
       wxGetApp().SendCommand(wxT("getvariable"), CompileHash("variable", wxT("$_REQUEST"), 0));
      break;
    case cmd_variablesgetfiles:
       wxGetApp().SendCommand(wxT("getvariable"), CompileHash("variable", wxT("$_FILES"), 0));
      break;
    case cmd_variablesgetpost:
       wxGetApp().SendCommand(wxT("getvariable"), CompileHash("variable", wxT("$_POST"), 0));
      break;
    case cmd_variablesgetspecified:

      teValue =
        new wxTextEntryDialog(
          this,
          "Specify what variable or define you wish to view (including $ for variables!)",
          "View variable or define",
          "");

      if(teValue->ShowModal() ==  wxID_OK)
      {
        if(teValue->GetValue() != "")
           wxGetApp().SendCommand(wxT("getvariable"), CompileHash("variable", teValue->GetValue().c_str(), 0));
      }
      break;
    case cmd_variablesgetexpression:

      teValue =
        new wxTextEntryDialog(
          this,
          "Specify what expression you wish to view",
          "View expression",
          "");

      if(teValue->ShowModal() ==  wxID_OK)
      {
        if(teValue->GetValue() != "")
           wxGetApp().SendCommand(wxT("getvariable"), CompileHash("variable", teValue->GetValue().c_str(), 0));
      }
      break;
  }

}
void frmGubed::OnCloseFile(wxCommandEvent& event)
{
  m_nbSourceLists->CloseSource();
}

void frmGubed::OnOpenFile(wxCommandEvent& event)
{
  wxConfigBase *cfgGubed = wxConfigBase::Get();

  wxFileDialog dlg(this, _("Open script file"), cfgGubed->Read(_T("GUI/OpenDialogDefaultDir"), _T("")), _T(""), _T("PHP Files|*.php;*.phtml;*.php?|All Files|*.*"), wxOPEN | wxFILE_MUST_EXIST | wxMULTIPLE);
  if(dlg.ShowModal() == wxID_OK)
  {
    wxArrayString filenames;
    dlg.GetPaths(filenames);

    PathMapperDialog mapper(this);
    if(mapper.ShowModal(dlg.GetDirectory()) == wxID_OK)
    {

      for(unsigned int cnt = 0; cnt < filenames.Count(); cnt++)
      {
        wxString filename = filenames.Item(cnt);

        wxTextFile file;
        file.Open(filename);
        if(file.IsOpened() && file.GetLineCount() > 0)
        {
          wxString contents, linestr;
          for ( linestr = file.GetFirstLine(); !file.Eof(); linestr = file.GetNextLine() )
            contents += linestr + "\n";

          m_nbSourceLists->AddSource(mapper.MapLocalToRemote(filename), _T(""));
          m_nbSourceLists->SetSource(mapper.MapLocalToRemote(filename), contents);
        }
        else
          wxLogWarning(wxString::Format(_("Unable to open file \"%s\"."), filename.c_str()));
      }
    }
    cfgGubed->Write(_T("GUI/OpenDialogDefaultDir"), dlg.GetDirectory());
    cfgGubed->Flush();
  }

}

void frmGubed::OnWatch(wxCommandEvent& event)
{
  wxTextEntryDialog *teValue;

  switch(event.GetId())
  {
    case cmd_watchgetserver:
       wxGetApp().AddWatch(wxT("$_SERVER"));
      break;
    case cmd_watchgetsession:
       wxGetApp().AddWatch(wxT("$_SESSION"));
      break;
    case cmd_watchgetenv:
       wxGetApp().AddWatch(wxT("$_ENV"));
      break;
    case cmd_watchgetthis:
       wxGetApp().AddWatch(wxT("$this"));
      break;
    case cmd_watchgetglobals:
       wxGetApp().AddWatch(wxT("$GLOBALS"));
      break;
    case cmd_watchgetget:
       wxGetApp().AddWatch(wxT("$_GET"));
      break;
    case cmd_watchgetrequest:
       wxGetApp().AddWatch(wxT("$_REQUEST"));
      break;
    case cmd_watchgetfiles:
       wxGetApp().AddWatch(wxT("$_FILES"));
      break;
    case cmd_watchgetpost:
        wxGetApp().AddWatch(wxT("$_POST"));
      break;
    case cmd_watchgetspecified:

      teValue =
        new wxTextEntryDialog(
          this,
          _("Specify what variable you wish to watch (including $)"),
          _("Watch variable"),
          _T(""));

      if(teValue->ShowModal() ==  wxID_OK)
      {
        if(teValue->GetValue() != "")
          wxGetApp().AddWatch(teValue->GetValue());
      }
      break;
    case cmd_watchgetexpression:

      teValue =
        new wxTextEntryDialog(
          this,
          _("Specify what expression you wish to watch"),
          _("Watch expression"),
          _T(""));

      if(teValue->ShowModal() ==  wxID_OK)
      {
        if(teValue->GetValue() != _T(""))
          wxGetApp().AddWatch(teValue->GetValue());
      }
      break;
  }
}


nbSources *frmGubed::GetSources()
{
  return m_nbSourceLists;
}

void frmGubed::DebugMode(bool blDebugMode)
{

  EnableCommand(cmd_stepinto, blDebugMode);
  EnableCommand(cmd_stepover, blDebugMode);
  EnableCommand(cmd_stepout, blDebugMode);
  EnableCommand(cmd_skip, blDebugMode);
  EnableCommand(cmd_kill, blDebugMode);
  EnableCommand(cmd_findsourceline, blDebugMode);

  wxGetApp().setDisplayDelay(-1);

}

void frmGubed::ShowState(ExecutionStates::State state)
{
  EnableCommand(cmd_pause, state != ExecutionStates::Pause);
  EnableCommand(cmd_trace, state != ExecutionStates::Trace);
  EnableCommand(cmd_run, state != ExecutionStates::Run);
}

void frmGubed::EnableCommand(int id, const bool enable)
{
  m_tbMainToolbar->EnableTool(id, enable);

  wxMenuBar * menuBar = GetMenuBar();
  if(menuBar)
    menuBar->Enable(id, enable);
}

void frmGubed::PrepareWatches(int intId, bool blAllSent)
{

  if(blAllSent)
  {
    m_intCurrentWatchId = intId;
    DeleteOldWatches(m_watchroot);
    DeleteOldWatches(m_viewroot);

  }
  else
    m_intCurrentWatchId = intId;
}

void frmGubed::DeleteOldWatches(wxTreeItemId tiCheck)
{
  long lngCookie;
  wxTreeItemId tiCurrent;
  tidWatchData *tidWatch;

  tiCurrent = m_tcWatches->GetFirstChild(tiCheck, lngCookie);
  while(tiCurrent)
  {
    tidWatch = (tidWatchData*)m_tcWatches->GetItemData(tiCurrent);

    if(m_tcWatches->ItemHasChildren(tiCurrent))
      DeleteOldWatches(tiCurrent);

    if(tidWatch->intWatchId != m_intCurrentWatchId)
    {
      const wxTreeItemId & tiNext = m_tcWatches->GetNextSibling(tiCurrent);;

      m_hmwWatches.erase(tidWatch->strWatch);


      m_tcWatches->Delete(tiCurrent);
      tiCurrent = tiNext;

    }
    else
      tiCurrent = m_tcWatches->GetNextSibling(tiCurrent);
  }
}

void frmGubed::ShowWatch(const wxString& strData)
{
  int intPos = 0;

  ParseWatches(strData, &intPos, 1, m_watchroot);
  m_tcWatches->Expand(m_watchroot);
}

void frmGubed::ShowBacktrace(const wxString& backtrace)
{
  wxString bt = backtrace;
  m_backtracelist->BeginBatch();
  
  // Remove current list
  if(m_backtracelist->GetNumberRows() > 0)
    m_backtracelist->DeleteRows(0, m_backtracelist->GetNumberRows());
  
  int frame;
  while(bt.Length() > 0)
  {
    wxString tmp, arg;
    tmp = bt.Mid(0, bt.Find(';'));
    frame = wxAtoi(tmp.AfterFirst(':'));
    
    // Add rows to grid
    if(m_backtracelist->GetNumberRows() <= frame)
      m_backtracelist->AppendRows(frame - m_backtracelist->GetNumberRows() + 1);  
    
    // Set the frame # in first column
    m_backtracelist->SetCellValue(frame, BacktraceColumns::Frame, wxString::Format(wxT("%d"), frame));
        
    bt = bt.AfterFirst('{');
     
    while(bt.Mid(0, 1) != wxT("}") && bt.Length() > 0)
    {
      int len = wxAtoi(bt.AfterFirst(':').BeforeFirst(':'));
      arg = bt.AfterFirst('"').Mid(0, len);
      bt = bt.AfterFirst(';');
      
      tmp = wxT("");
      if(bt.Mid(0, 1) == wxT("s"))
      {
        int len = wxAtoi(bt.AfterFirst(':').BeforeFirst(':'));
        bt = bt.AfterFirst('"');
        tmp = bt.Mid(0, len);
        bt = bt.Mid(len + 2);
      }
      else
      {
        tmp = bt.BeforeFirst(';').AfterFirst(':');
        bt = bt.AfterFirst(';');
      }
    
      if(arg == wxT("function"))
        m_backtracelist->SetCellValue(frame, BacktraceColumns::Function, tmp + wxT(" "));
      else if(arg == wxT("class"))
        m_backtracelist->SetCellValue(frame, BacktraceColumns::Class, tmp + wxT(" "));
      else if(arg == wxT("file"))
      {
        m_backtracelist->SetCellValue(frame, BacktraceColumns::Filename, tmp + wxT(" "));
        m_backtracelist->SetCellAlignment(wxALIGN_RIGHT, frame, BacktraceColumns::Filename);
      }
      else if(arg == wxT("line"))
      {
        m_backtracelist->SetCellValue(frame, BacktraceColumns::Line, tmp + wxT(" "));
        m_backtracelist->SetCellAlignment(wxALIGN_RIGHT, frame, BacktraceColumns::Line);
      } 
    } 
  }
      
  //  i:0;a:4:{s:4:"file";s:82:"classes/baseclass.php";s:5:"class";s:9:"baseclass";s:8:"function";s:5:"func1";s:4:"line";i:16;}i:1;a:4:{s:4:"file";s:77:"/classes/...
  
  //wxString asdf   
  
  m_backtracelist->EndBatch();

}


void frmGubed::updateBreakpoint(int idx, const Breakpoint &bp)
{
  // Make sure we have enough lines
  if(m_breakpointlist->GetNumberRows() <= idx)
    m_breakpointlist->AppendRows(idx - m_breakpointlist->GetNumberRows() + 1);
  
  m_breakpointlist->SetCellValue(idx, BreakpointColumns::Type, bp.getTypeName());
  m_breakpointlist->SetCellValue(idx, BreakpointColumns::Filename, bp.getFilename());
  m_breakpointlist->SetCellAlignment(wxALIGN_RIGHT, idx, BreakpointColumns::Filename);
  m_breakpointlist->SetCellValue(idx, BreakpointColumns::Line, wxString::Format(wxT("%ld"), 1 + bp.getLine()));
  m_breakpointlist->SetCellAlignment(wxALIGN_RIGHT, idx, BreakpointColumns::Line);
  m_breakpointlist->SetCellValue(idx, BreakpointColumns::Expression, bp.getExpression());
  m_breakpointlist->SetCellValue(idx, BreakpointColumns::Class, bp.getClass());
  m_breakpointlist->SetCellValue(idx, BreakpointColumns::Function, bp.getFunction());
  m_breakpointlist->SetCellValue(idx, BreakpointColumns::Value, bp.getValue());
}

void frmGubed::removeBreakpoint(int idx)
{
  m_breakpointlist->DeleteRows(idx, 1);
}

void frmGubed::ShowVariable(const wxString& strData)
{
  int intPos = 0;

  wxString strTip = ParseWatches(strData,  &intPos, 1, m_viewroot);

  // This doesent work that great with stc (it starts to mark text)
  #ifndef HAVE_STC
    PopTip(strTip);
  #endif

  m_tcWatches->Expand(m_viewroot);

}

void frmGubed::PopTip(const wxString & strTip)
{

  if(m_tipwindow)
    m_tipwindow->Destroy();

  wxRect* rect = new wxRect();
  rect->Offset(::wxGetMousePosition());
  rect->Offset(-30, -30);
  rect->Inflate(60, 60);
  m_tipwindow = new wxTipWindow(this, strTip, 100L, &m_tipwindow, rect);
}

wxString frmGubed::ParseWatches(const wxString& strData, int* pintPos, int intLevel, const wxTreeItemId& tiCurrentRoot)
{

  wxString strPart, strType, strKey, strReturn;
  long lngLength;
  
  while((ulong)*pintPos < strData.size())
  {
    // Read type of key
    strType = strData.substr(*pintPos, 1);

    if(strType == "}")
    {
      *pintPos += 1;
      return strReturn;
    }

    //Log("strType 1: " + strType);
    *pintPos += 2;

    if(strType == ";")
    {
      strType = strData.substr(*pintPos, 1);
      *pintPos += 1;
    }
    //Log("strType 2: " + strType);

    if(strType == "s")
    {
      strPart = strData.substr(*pintPos, strData.find(":", *pintPos) - *pintPos);
      strPart.ToLong(&lngLength);
      *pintPos += strPart.size() + 2;

      strKey = strData.substr(*pintPos, lngLength);
      *pintPos += lngLength + 2;

    }
    else if(strType == "i")
    {
      strKey = strData.substr(*pintPos, strData.find(";", *pintPos) - *pintPos);
      *pintPos += strKey.size() + 1;

    }
    else
      return strReturn;

    strType = strData.substr(*pintPos, 1);
    *pintPos += 2;
    strPart = strData.substr(*pintPos);

    // String
    if(strType == "s")
    {
      *pintPos += strPart.find(":") + 2;

      strPart = strPart.substr(0, strPart.find(":"));
      strPart.ToLong(&lngLength);

      strPart = strData.substr(*pintPos, lngLength);

      ReplaceWatch(strKey, strType, intLevel, lngLength, strPart, tiCurrentRoot);
      strReturn = strPart;

      *pintPos += strPart.size() + 2;


    }
    // Number
    else if(strType == "i" || strType == "d")
    {

      *pintPos += strPart.find_first_not_of("0123456789.,-") + 1;
      strPart = strPart.substr(0, strPart.find_first_not_of("0123456789.,-"));

      ReplaceWatch(strKey, strType, intLevel, -1, strPart, tiCurrentRoot);
      strReturn = strPart;

    }
    // Array
    else if(strType == "a")
    {
      long lngSize;

      *pintPos += strPart.find(":") + 2;
      strPart.ToLong(&lngSize);


      strPart = lngSize == 0 ? _("<Empty array>"): _("<Array>");
      const wxTreeItemId & tiArray = ReplaceWatch(strKey, strType, intLevel, -1, strPart, tiCurrentRoot);
      strReturn = strPart;

      intLevel++;

      ParseWatches(strData, pintPos, intLevel, tiArray);
      intLevel--;

    }
    // Object
    else if(strType == "O")
    {
      wxString strObj;

      *pintPos += strPart.find(":") + 2;

      strObj = strPart.substr(0, strPart.find(":"));
      strObj.ToLong(&lngLength);

      strObj = _("Object: ") + strData.substr(*pintPos, lngLength);
      strReturn = strObj;

      *pintPos += lngLength + 1;
      *pintPos += strPart.find(":") + 3;


      const wxTreeItemId & tiObject = ReplaceWatch(strKey, strType, intLevel, -1, strObj, tiCurrentRoot);
      intLevel++;

      ParseWatches(strData, pintPos, intLevel, tiObject);
      intLevel--;

    }
    // Boolean
    else if(strType == "b")
    {

      *pintPos += 2;
      strPart = strPart.substr(0, 1);
      strPart = (strPart == "0" ? _("false") : _("true"));

      ReplaceWatch(strKey, strType, intLevel, 0, strPart, tiCurrentRoot);
      strReturn = strPart;

    }
    // NUll, Example: s:6:"return";N;
    else if(strType == "N")
    {
      ReplaceWatch(strKey, strType, intLevel, -1, _("<NULL>"), tiCurrentRoot);
      strReturn = _("<NULL>");

    }
    // * - Shows text 'changed' or blank (used for conditional bps)
    else if(strType == wxT("*"))
    {
      *pintPos += 2;
      strPart = strPart.substr(0, 1);
      strPart = (strPart == "+" ? _("Fulfilled") : wxT(""));

      ReplaceWatch(strKey, strType, intLevel, -1, strPart, tiCurrentRoot);
      strReturn = strPart;

    }
    // Unavailable
    else if(strType == "-")
    {
      ReplaceWatch(strKey, strType, intLevel, -1, _("<Unavailable>"), tiCurrentRoot);
      strReturn = _("<Unavailable>");

    }
    // Error
    else if(strType == "!")
    {
      ReplaceWatch(strKey, strType, intLevel, -1, _("<Error>"), tiCurrentRoot);
      strReturn = _("<Error>");

    }
    else
    {
      ReplaceWatch(strKey, strType, intLevel, 0, _("<Unknown type: ") + strType + ">", tiCurrentRoot);
      strReturn =  _("<Unknown type: ") + strType + ">";

    }
  }
  return strReturn;
}

const wxTreeItemId & frmGubed::ReplaceWatch(const wxString& strKey, const wxString& strType, int intLevel, int lngSize, const wxString& strData, const wxTreeItemId &tiCurrentRoot)
{

  wxString strWatch, strAdd, strTmp;

  if(lngSize >= 0)
    strAdd << strKey << " (" << lngSize << "): ";
  else
    strAdd = strKey + ": ";
  strAdd += strData;

  strWatch << (long)tiCurrentRoot << "_" << strKey;

  if(m_hmwWatches.find(strWatch) == m_hmwWatches.end())
  {
    wxTreeItemId tiAdd = m_tcWatches->AppendItem(tiCurrentRoot, strAdd);
    m_hmwWatches[strWatch] = tiAdd;
  }
  else
    m_tcWatches->SetItemText(m_hmwWatches[strWatch], strAdd);

  tidWatchData *tidData = new tidWatchData();
  tidData->intWatchId = m_intCurrentWatchId;
  tidData->strVariable = strKey;
  tidData->strWatch = strWatch;
  m_tcWatches->SetItemData(m_hmwWatches[strWatch], tidData);

  return m_hmwWatches[strWatch];

}

void frmGubed::SetupMenues()
{
  wxMenuBar *menuBar = new wxMenuBar;

  wxMenu *menuTmp = new wxMenu;
  wxMenuItem *mnuItem;

  mnuItem = new wxMenuItem(menuTmp, cmd_openfile, _("&Open source file...\tCtrl-O" ), _("Open source file"));
  mnuItem->SetBitmap( wxBitmap(openfile_xpm));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_closefile, _("&Close current file\tCtrl-W" ), _("Close the current file"));
  mnuItem->SetBitmap( wxBitmap(closefile_xpm));
  menuTmp->Append(mnuItem);
  mnuItem->Enable(false);

  menuTmp->AppendSeparator();


  mnuItem = new wxMenuItem(menuTmp, cmd_connect, _("&Connect" ), _("Connect"));
  mnuItem->SetBitmap( wxBitmap(connect_xpm));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_disconnect, _("&Disconnect" ), _("Disconnect from Gubed proxy"));
  mnuItem->SetBitmap( wxBitmap(disconnect_xpm));
  menuTmp->Append(mnuItem);
  //menuTmp->Append( cmd_connect, _("&Connect"));

  menuTmp->AppendSeparator();
  mnuItem = new wxMenuItem(menuTmp, cmd_settings, _("&Settings...\tCtrl-Shift-O" ), _("View and edit settings"));
  //mnuItem->SetBitmap( wxBitmap(connect_xpm));menuTmp->Append(mnuItem);
  menuTmp->Append(mnuItem);
  //menuTmp->Append( cmd_settings, _("&Settings..."));

  menuTmp->AppendSeparator();
  mnuItem = new wxMenuItem(menuTmp, cmd_quit, _("&Quit\tCtrl-Q" ), _("Quit client"));
  //mnuItem->SetBitmap( wxBitmap(connect_xpm));
  menuTmp->Append(mnuItem);
  //menuTmp->Append( cmd_quit, _("E&xit" ));
  menuBar->Append( menuTmp, _("&Program"));

  menuTmp = new wxMenu;

  mnuItem = new wxMenuItem(menuTmp, cmd_trace, _("&Trace" ), _("Trace throught the script"));
  mnuItem->SetBitmap( wxBitmap(trace_xpm));
  menuTmp->Append(mnuItem);
  //menuTmp->Append(cmd_trace, _("Trace"));
  mnuItem = new wxMenuItem(menuTmp, cmd_run, _("&Run\tCtrl-R" ), _("Run script"));
  mnuItem->SetBitmap( wxBitmap(run_xpm));
  menuTmp->Append(mnuItem);
  //menuTmp->Append(cmd_run, _("Run"));
  mnuItem = new wxMenuItem(menuTmp, cmd_pause, _("&Pause\tCtrl-P" ), _("Pause execution"));
  mnuItem->SetBitmap( wxBitmap(pause_xpm));
  menuTmp->Append(mnuItem);
  //menuTmp->Append(cmd_pause, _("Pause"));
  menuTmp->AppendSeparator();
  mnuItem = new wxMenuItem(menuTmp, cmd_kill, _("&Kill\tCtrl-K" ), _("Kill sctipt"));
  mnuItem->SetBitmap( wxBitmap(kill_xpm));
  menuTmp->Append(mnuItem);
  //menuTmp->Append(cmd_kill, _("Kill"));
  menuTmp->AppendSeparator();
  mnuItem = new wxMenuItem(menuTmp, cmd_stepover, _("&Step Over\tCtrl-1" ), _("Step over function"));
  mnuItem->SetBitmap( wxBitmap(stepover_xpm));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_stepinto, _("&Step Into\tCtrl-2" ), _("Step into function"));
  mnuItem->SetBitmap( wxBitmap(stepinto_xpm));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_stepout, _("&Step Out\tCtrl-3" ), _("Step out of function"));
  mnuItem->SetBitmap( wxBitmap(stepout_xpm));
  menuTmp->Append(mnuItem);
  //menuTmp->Append(cmd_step, _("Step"));
  mnuItem = new wxMenuItem(menuTmp, cmd_skip, _("Skip\tCtrl-4" ), _("Skip next instruction"));
  mnuItem->SetBitmap( wxBitmap(skip_xpm));
  menuTmp->Append(mnuItem);
  //menuTmp->Append(cmd_skip, _("Skip"));

  menuBar->Append( menuTmp, _("&Control"));

  menuTmp = new wxMenu;
  mnuItem = new wxMenuItem(menuTmp, cmd_variablesgetall, _("Current scope\tCtrl-E" ), _("Browse all variables in current scope"));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_variablesgetspecified, _("Variable..." ), _("Specify variable to view"));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_variablesgetexpression, _("Expression..." ), _("Specify variable to view"));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_variablesgetthis, _("$this" ), _("View the contents of '$this'"));
  menuTmp->Append(mnuItem);

  menuTmp->AppendSeparator();
  mnuItem = new wxMenuItem(menuTmp, cmd_variablesgetserver, _("$_SERVER" ), _("View the contents of '$_SERVER'"));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_variablesgetenv, _("$_ENV" ), _("View the contents of '$_ENV'"));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_variablesgetsession, _("$_SESSION" ), _("View the contents of '$_SESSION'"));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_variablesgetglobals, _("$GLOBALS" ), _("View the contents of '$GLOBALS'"));
  menuTmp->Append(mnuItem);

  menuTmp->AppendSeparator();
  mnuItem = new wxMenuItem(menuTmp, cmd_variablesgetrequest, _("$_REQUEST" ), _("View the contents of '$_REQUEST'"));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_variablesgetget, _("$_GET" ), _("View the contents of '$_GET'"));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_variablesgetpost, _("$_POST" ), _("View the contents of '$_POST'"));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_variablesgetfiles, _("$_FILES" ), _("View the contents of '$_FILES'"));
  menuTmp->Append(mnuItem);

  menuBar->Append( menuTmp, _("&Get..."));



  menuTmp = new wxMenu;
  mnuItem = new wxMenuItem(menuTmp, cmd_watchgetspecified, _("Variable...\tCtrl-K" ), _("Specify variable to watch"));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_watchgetexpression, _("Expression...\tCtrl-H" ), _("Specify expression to watch"));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_watchgetthis, _("$this\tCtrl-G" ), _("Watch the contents of '$this'"));
  menuTmp->Append(mnuItem);

  menuTmp->AppendSeparator();
  mnuItem = new wxMenuItem(menuTmp, cmd_watchgetserver, _("$_SERVER" ), _("Watch the contents of '$_SERVER'"));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_watchgetenv, _("$_ENV" ), _("Watch the contents of '$_ENV'"));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_watchgetsession, _("$_SESSION" ), _("Watch the contents of '$_SESSION'"));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_watchgetglobals, _("$GLOBALS" ), _("Watch the contents of '$GLOBALS'"));
  menuTmp->Append(mnuItem);

  menuTmp->AppendSeparator();
  mnuItem = new wxMenuItem(menuTmp, cmd_watchgetrequest, _("$_REQUEST" ), _("Watch the contents of '$_REQUEST'"));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_watchgetget, _("$_GET" ), _("Watch the contents of '$_GET'"));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_watchgetpost, _("$_POST" ), _("Watch the contents of '$_POST'"));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_watchgetfiles, _("$_FILES" ), _("Watch the contents of '$_FILES'"));
  menuTmp->Append(mnuItem);

  menuBar->Append( menuTmp, _("&Watch..."));


  menuTmp = new wxMenu;
  mnuItem = new wxMenuItem(menuTmp, cmd_conditionalbreakpoint, _("Add conditional\tCtrl-N" ), _("Add conditional breakpoint"));
  //mnuItem->SetBitmap( wxBitmap(breakpoint_xpm));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_breakpoint, _("Toggle\tCtrl-M" ), _("Toggle breakpoint"));
  mnuItem->SetBitmap( wxBitmap(breakpoint_xpm));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_nextbreakpoint, _("Next\tCtrl-." ), _("Find next breakpoint"));
  mnuItem->SetBitmap( wxBitmap(next_xpm));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_prevbreakpoint, _("Previous\tCtrl-," ), _("Find previous breakpoint"));
  mnuItem->SetBitmap( wxBitmap(prev_xpm));
  menuTmp->Append(mnuItem);

  menuBar->Append( menuTmp, _("&Breakpoint" ));

  menuTmp = new wxMenu;
  mnuItem = new wxMenuItem(menuTmp, cmd_findsourceline, _("Current line" ), _("Jump to current line of execution"));
  mnuItem->SetBitmap( wxBitmap(currentline_xpm));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_bookmark, _("Toggle\tCtrl-B" ), _("Toggle bookmark"));
  mnuItem->SetBitmap( wxBitmap(bookmark_xpm));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_nextbookmark, _("Next\tCtrl-0" ), _("Find next bookmark"));
  mnuItem->SetBitmap( wxBitmap(next_xpm));
  menuTmp->Append(mnuItem);
  mnuItem = new wxMenuItem(menuTmp, cmd_prevbookmark, _("Previous\tCtrl-9" ), _("Find previous bookmark"));
  mnuItem->SetBitmap( wxBitmap(prev_xpm));
  menuTmp->Append(mnuItem);

  menuBar->Append( menuTmp, _("Bookmar&k" ));

  menuTmp = new wxMenu;
  //mnuItem = new wxMenuItem(menuTmp, );
  menuTmp->AppendCheckItem(cmd_logverbose, _("Verbose log" ), _("Log more details"));
  #ifdef __WXDEBUG__
    menuTmp->AppendCheckItem(cmd_logdebug, _("Debug log" ), _("Log debug messages"));
  #endif
  
  mnuItem = new wxMenuItem(menuTmp, cmd_logclear, _("Cle&ar log\tCtrl-A" ), _("Clear log window"));
  //mnuItem->SetBitmap( wxBitmap(connect_xpm));menuTmp->Append(mnuItem);
  menuTmp->Append(mnuItem);

  mnuItem = new wxMenuItem(menuTmp, cmd_logsave, _("&Save\tCtrl-S" ), _("Save log output"));
  //mnuItem->SetBitmap( wxBitmap(next_xpm));
  menuTmp->Append(mnuItem);

  menuBar->Append( menuTmp, _("&Log" ));



  menuTmp = new wxMenu;
  menuTmp->Append(cmd_about, _("About"));
  menuTmp->Append(cmd_help, _("Help"));

  menuBar->Append( menuTmp, _("&Help"));

  SetMenuBar( menuBar );
}


void frmGubed::SetupToolbar()
{
  int intToolbar;
  long lngStyle = 0;
  wxConfigBase *cfgGubed = wxConfigBase::Get();

  if(m_tbMainToolbar)
  {
    SetToolBar(NULL);
    delete m_tbMainToolbar;
    m_tbMainToolbar = NULL;
  }

  intToolbar = cfgGubed->Read(wxT("Toolbar"), 3l);
  //{_("No toolbar"), _("Images only"), _("Text Only"), _("Images and text")};
  switch(intToolbar)
  {
    case 0:
      return;

    case 1:
      lngStyle = wxTB_HORIZONTAL | wxTB_DOCKABLE | wxNO_BORDER | wxTB_FLAT;
      break;

    case 2:
      lngStyle = wxTB_HORIZONTAL | wxTB_DOCKABLE | wxTB_TEXT | wxTB_NOICONS | wxNO_BORDER | wxTB_FLAT;
      break;

    case 3:
      lngStyle = wxTB_HORIZONTAL | wxTB_DOCKABLE | wxTB_TEXT | wxNO_BORDER | wxTB_FLAT;
      break;
  }
  m_tbMainToolbar = CreateToolBar(lngStyle);

  if(m_tbMainToolbar)
  {
    m_tbMainToolbar->SetToolBitmapSize(wxSize(16,16));
    m_tbMainToolbar->AddTool(cmd_quit, _("Quit"), wxBitmap(quit_xpm), _("Quit"));
    m_tbMainToolbar->AddSeparator();
    m_tbMainToolbar->AddSeparator();
    m_tbMainToolbar->AddTool(cmd_connect, _("Connect"), wxBitmap(connect_xpm),_("Connect"));
    m_tbMainToolbar->AddTool(cmd_disconnect, _("Disconnect"), wxBitmap(disconnect_xpm),_("Disonnect"));
    m_tbMainToolbar->AddSeparator();
    m_tbMainToolbar->AddSeparator();
    m_tbMainToolbar->AddTool(cmd_trace, _("Trace"), wxBitmap(trace_xpm), _("Executes a debugged script step by step, updating watches and current line after every command. If a script is currently not debugged, the next one will be started in trace mode."));
    m_tbMainToolbar->AddTool(cmd_run, _("Run"), wxBitmap(run_xpm), _("Executes a debugged script as quickly as possible, not updating watches and current until the script is paused. If a script is currently not debugged, the next one will be started in run mode."));
    m_tbMainToolbar->AddTool(cmd_pause, _("Pause"), wxBitmap(pause_xpm),_("Pause a currently executing script. If a script is currently not debugged, the next one will be started in paused mode."));
    m_tbMainToolbar->AddSeparator();
    m_tbMainToolbar->AddTool(cmd_kill, _("Kill"), wxBitmap(kill_xpm), _("Kill the currently running script."));
    m_tbMainToolbar->AddSeparator();
    m_tbMainToolbar->AddTool(cmd_stepover, _("Step Over"), wxBitmap(stepover_xpm), _("Execute next command but do not step into it if it is a function call or inclusion of a file"));
    m_tbMainToolbar->AddTool(cmd_stepinto, _("Step Into"), wxBitmap(stepinto_xpm), _("Execute next command and step into it if it is a function call or inclusion of a file"));
    m_tbMainToolbar->AddTool(cmd_stepout, _("Step Out"), wxBitmap(stepout_xpm), _("Step out of function, run until we're back to where we were called from"));
    m_tbMainToolbar->AddTool(cmd_skip, _("Skip"), wxBitmap(skip_xpm), _("Skip"));
    m_tbMainToolbar->AddTool(cmd_findsourceline, _("Current line"), wxBitmap(currentline_xpm), _("Show current line"));
    m_tbMainToolbar->AddSeparator();
    m_tbMainToolbar->AddSeparator();

    m_tbMainToolbar->AddTool(cmd_prevbreakpoint, _("Prev"), wxBitmap(prev_xpm), _("Go to previous Breakpoint"));
    m_tbMainToolbar->AddTool(cmd_breakpoint, _("Breakpoint"), wxBitmap(breakpoint_xpm), _("Toggle Breakpoint"));
    m_tbMainToolbar->AddTool(cmd_nextbreakpoint, _("Next"), wxBitmap(next_xpm), _("Go to next Breakpoint"));
    m_tbMainToolbar->AddSeparator();
    m_tbMainToolbar->AddTool(cmd_prevbookmark, _("Prev"), wxBitmap(prev_xpm), _("Go to previous Bookmark"));
    m_tbMainToolbar->AddTool(cmd_bookmark, _("Bookmark"), wxBitmap(bookmark_xpm), _("Toggle Bookmark"));
    m_tbMainToolbar->AddTool(cmd_nextbookmark, _("Next"), wxBitmap(next_xpm), _("Go to next Bookmark"));

    m_tbMainToolbar->AddSeparator();
    m_tbMainToolbar->AddControl(m_delayslider = new wxSlider(m_tbMainToolbar, cmd_tracedelay, wxGetApp().displayDelay(), 0, 200, wxDefaultPosition, wxSize(150, 24)));
    m_tbMainToolbar->Realize();
  }
}

void frmGubed::SetupControls()
{

  int intHorizPos, intVertPos;
  wxConfigBase *cfgGubed = wxConfigBase::Get();

  intHorizPos = cfgGubed->Read(wxT("WindowHorizontalSplit"), 130l);
  intVertPos = cfgGubed->Read(wxT("WindowVerticalSplit"), -130l);

  wxBoxSizer *bsWindow = new wxBoxSizer(wxHORIZONTAL);

  m_swHorizontal = new wxSplitterWindow(this, -1, wxDefaultPosition, wxDefaultSize);
  m_swVertical = new wxSplitterWindow(m_swHorizontal, -1, wxDefaultPosition, wxDefaultSize);
  m_swHorizontal->SetMinimumPaneSize(1);
  m_swVertical->SetMinimumPaneSize(1);

  // Watch tree
  m_tcWatches = new wxTreeCtrl(m_swVertical, -1, wxDefaultPosition, wxDefaultSize, wxTR_HIDE_ROOT | wxTR_HAS_BUTTONS /* | wxTR_NO_LINES*/);

  if(!m_tcWatches->GetRootItem())
    m_tcWatches->AddRoot(_("Watches"), -1, -1, new tidWatchData());

  // Source view
  m_nbSourceLists = new nbSources(m_swVertical, (cfgGubed->Read(wxT("TabsLeft"), 0l) ? wxNB_LEFT : 0) | wxNB_MULTILINE );

    
  // Bottom tab
  wxNotebook *bottomtab = new wxNotebook(m_swHorizontal, -1);

  // Log tab  
  wxSplitterWindow *logsplit = new wxSplitterWindow(bottomtab, -1, wxDefaultPosition, wxDefaultSize);
  
  m_txtLog = new wxTextCtrl(logsplit, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE | wxTE_READONLY);
  wxLog *logLogger = new wxLogTextCtrl(m_txtLog);
  wxLog::SetActiveTarget(logLogger);
  
  wxPanel* commandpanel = new wxPanel(logsplit);
  m_txtCommand = new wxTextCtrl(commandpanel, cmd_commandtext, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE);
  m_btnCommand = new wxButton(commandpanel, cmd_command, _("Process"));
  wxBoxSizer *commandsizer = new wxBoxSizer(wxHORIZONTAL);
  commandsizer->Add(m_txtCommand, 1, wxEXPAND );
  commandsizer->Add(m_btnCommand, 0, wxEXPAND );
  commandpanel->SetSizer(commandsizer);
  
  int height;
  m_txtCommand->GetTextExtent(_T("M"), 0, &height);
  logsplit->SetMinimumPaneSize(1);
  logsplit->SplitHorizontally(m_txtLog, commandpanel, -height * 4);
  //logsplit->SetSashGravity(0.9); // wx 2.5
  
  bottomtab->AddPage(logsplit, _("Log"), true);  
  
  // Breakpoint tab
  m_breakpointlist = new wxGrid(bottomtab, grd_breakpointlist);
  bottomtab->AddPage(m_breakpointlist, _("Breakpoints"), false);  
  m_breakpointlist->SetRowLabelSize(0);
  m_breakpointlist->EnableEditing(false);
  m_breakpointlist->SetGridLineColour(wxColor(155, 155, 155));
  m_breakpointlist->CreateGrid(0, BreakpointColumns::ColumnCount, wxGrid::wxGridSelectRows);
  m_breakpointlist->SetColLabelValue(BreakpointColumns::Type, _("Type"));
  m_breakpointlist->SetColLabelValue(BreakpointColumns::Filename, _("File"));
  m_breakpointlist->SetColLabelValue(BreakpointColumns::Line, _("Line"));
  m_breakpointlist->SetColLabelValue(BreakpointColumns::Expression, _("Expression"));
  m_breakpointlist->SetColLabelValue(BreakpointColumns::Class, _("Class"));
  m_breakpointlist->SetColLabelValue(BreakpointColumns::Function, _("Function"));
  m_breakpointlist->SetColLabelValue(BreakpointColumns::Value, _("Value"));

  // Breakpoint tab
  m_backtracelist = new wxGrid(bottomtab, grd_backtracelist);
  bottomtab->AddPage(m_backtracelist, _("Backtrace"), false);  
  m_backtracelist->SetRowLabelSize(0);
  m_backtracelist->EnableEditing(false);
  m_backtracelist->SetGridLineColour(wxColor(155, 155, 155));
  m_backtracelist->CreateGrid(0, BacktraceColumns::ColumnCount, wxGrid::wxGridSelectRows);
  m_backtracelist->SetColLabelValue(BacktraceColumns::Frame, _("Frame"));
  m_backtracelist->SetColLabelValue(BacktraceColumns::Filename, _("File"));
  m_backtracelist->SetColLabelValue(BacktraceColumns::Class, _("Class"));
  m_backtracelist->SetColLabelValue(BacktraceColumns::Function, _("Function"));
  m_backtracelist->SetColLabelValue(BacktraceColumns::Line, _("Line"));
    
  // Window layout
  m_swVertical->SplitVertically(m_tcWatches, m_nbSourceLists, intVertPos);
  m_swHorizontal->SplitHorizontally(m_swVertical, bottomtab, intHorizPos);

  bsWindow->Add(m_swHorizontal, 1, wxEXPAND);

  SetSizer(bsWindow);
}


frmGubed::~frmGubed()
{
  wxLog::SetActiveTarget(new wxLogStderr);
  //wxConfig *cfgGubed = new wxConfig("Gubed");
  wxConfigBase *cfgGubed = wxConfigBase::Get();

  wxRect rctPos = this->GetRect();
  cfgGubed->Write(wxT("WindowLeft"), rctPos.GetLeft());
  cfgGubed->Write(wxT("WindowTop"), rctPos.GetTop());
  cfgGubed->Write(wxT("WindowWidth"), rctPos.GetWidth());
  cfgGubed->Write(wxT("WindowHeight"), rctPos.GetHeight());

  cfgGubed->Write(wxT("WindowHorizontalSplit"), m_swHorizontal->GetSashPosition());
  cfgGubed->Write(wxT("WindowVerticalSplit"), m_swVertical->GetSashPosition());
  cfgGubed->Flush();
}


void frmGubed::OnReSize(wxSizeEvent& evt)
{
  if(m_breakpointlist)
  {
     m_breakpointlist->SetColumnWidth(BreakpointColumns::Type, (int)( evt.GetSize().GetWidth() / 100.0 * 5.0));
     m_breakpointlist->SetColumnWidth(BreakpointColumns::Filename, (int)( evt.GetSize().GetWidth() / 100.0 * 20.0));
     m_breakpointlist->SetColumnWidth(BreakpointColumns::Line, (int)( evt.GetSize().GetWidth() / 100.0 * 5.0));
     m_breakpointlist->SetColumnWidth(BreakpointColumns::Expression, (int)( evt.GetSize().GetWidth() / 100.0 * 20.0));
     m_breakpointlist->SetColumnWidth(BreakpointColumns::Class, (int)( evt.GetSize().GetWidth() / 100.0 * 20.0));
     m_breakpointlist->SetColumnWidth(BreakpointColumns::Function, (int)( evt.GetSize().GetWidth() / 100.0 * 20.0));
     m_breakpointlist->SetColumnWidth(BreakpointColumns::Value, (int)( evt.GetSize().GetWidth() / 100.0 * 10.0));
  }
  
  if(m_backtracelist)
  {
    m_backtracelist->SetColumnWidth(BacktraceColumns::Frame, (int)( evt.GetSize().GetWidth() / 100.0 * 5.0));
    m_backtracelist->SetColumnWidth(BacktraceColumns::Filename, (int)(evt.GetSize().GetWidth() / 100.0 * 40.0));
    m_backtracelist->SetColumnWidth(BacktraceColumns::Class, (int)(evt.GetSize().GetWidth() / 100.0 * 25.0));
    m_backtracelist->SetColumnWidth(BacktraceColumns::Function, (int)(evt.GetSize().GetWidth() / 100.0 * 25.0));
    m_backtracelist->SetColumnWidth(BacktraceColumns::Line, (int)(evt.GetSize().GetWidth() / 100.0 * 5.0));
  }
  evt.Skip(true);  
}


