/*
 * Copyright (C) 2004 by Linus McCabe
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#ifndef FRMGUBED_H
#define FRMGUBED_H

#include "wxClientGlobal.h"
#include <wx/wx.h>
#include <wx/splitter.h>
#include <wx/treectrl.h>
#include <wx/socket.h>
#include <wx/tipwin.h>
#include <wx/slider.h>
#include <wx/toolbar.h>
#include <wx/grid.h>
#include "wx/html/helpfrm.h"
#include "wx/html/helpctrl.h"


#include "sockLineSocket.h"
#include "nbSources.h"
#include "Breakpoint.h"

WX_DECLARE_STRING_HASH_MAP(wxTreeItemId,  hmWatches );

namespace BreakpointColumns  
{
  enum Columns
  {
    Type = 0,
    Filename,
    Line,
    Expression,
    Class,
    Function,
    Value,
    ColumnCount
  };
}

namespace BacktraceColumns  
{  
  enum Columns 
  {
    Frame = 0,
    Filename,
    Class,
    Function,
    Line,
    ColumnCount
  };
}

class frmGubed: public wxFrame {
   private:   
   
      wxToolBar *m_tbMainToolbar; 
      nbSources *m_nbSourceLists;
      
      wxTextCtrl *m_txtLog;
      wxTextCtrl *m_txtCommand;
      wxButton *m_btnCommand;
      
      wxTreeCtrl *m_tcWatches;
      hmWatches m_hmwWatches;
      int m_intCurrentWatchId;
      wxTreeItemId m_watchroot;
      wxTreeItemId m_viewroot;

      wxSplitterWindow *m_swHorizontal;
      wxSplitterWindow *m_swVertical;

      wxGrid *m_breakpointlist;
      wxGrid *m_backtracelist;
      
      //frmSettings *frmSet;
      wxTipWindow *m_tipwindow;
      wxSlider * m_delayslider;

      // Help control
      wxHtmlHelpController m_help;
      
      wxString ParseWatches(const wxString&, int*, int, const wxTreeItemId&);
      const wxTreeItemId & ReplaceWatch(const wxString&, const wxString&, int, int, const wxString&, const wxTreeItemId& );
      void DeleteOldWatches(wxTreeItemId);

      void SetupMenues(void);
      void SetupToolbar(void);
      void SetupControls(void);
      void PopTip(const wxString &);

   public:
      frmGubed(const wxString& title, const wxPoint& pos, const wxSize& size);
      ~frmGubed(void);

      void PrepareWatches(int, bool);
      void ShowWatch(const wxString&);
      void ShowBacktrace(const wxString& backtrace);
      void ShowVariable(const wxString&);
     // void ShowCondition(const wxString&);
      void ShowState(ExecutionStates::State);
      void setConnectionState();

      void OnQuit(wxCommandEvent& event);
      void OnAbout(wxCommandEvent& event);
      void OnHelp(wxCommandEvent& event);
      void OnSettings(wxCommandEvent& event);
      void OnCommand(wxCommandEvent& event);
      void OnBookmark(wxCommandEvent& event);
      void OnNextBookmark(wxCommandEvent& event);
      void OnBreakpoint(wxCommandEvent& event);
      void OnConnect(wxCommandEvent& event);
      void OnWatchKeyDown(wxTreeEvent&);
      void OnScroll(wxScrollEvent&);
      void OnVariables(wxCommandEvent&);
      void OnWatch(wxCommandEvent&);
      void OnLogEnter(wxCommandEvent& event);
      
      void OnGridRightClick(wxGridEvent& event);      
      void OnGridDbl(wxGridEvent &evt);
      void OnReSize(wxSizeEvent&);

      void OnOpenFile(wxCommandEvent&);
      void OnCloseFile(wxCommandEvent&);

      void OnLog(wxCommandEvent&);

      void updateBreakpoint(int idx, const Breakpoint &bp);
      void removeBreakpoint(int idx);
      
      void EnableCommand(int id, const bool enable);
      void DebugMode(bool);
      nbSources *GetSources();

      DECLARE_EVENT_TABLE();
};

enum {
   cmd_quit = 5999,
   cmd_about,
   cmd_help,
   cmd_openfile,
   cmd_closefile,
   cmd_settings,
   cmd_connect,
   cmd_disconnect,

   cmd_controlsfirst,
   cmd_run,
   cmd_trace,
   cmd_pause,
   cmd_stepinto,
   cmd_stepover,
   cmd_stepout,
   cmd_skip,
   cmd_kill,
   cmd_findsourceline,
   cmd_controlslast,

   cmd_bookmarkFirst,
   cmd_bookmark,
   cmd_nextbookmark,
   cmd_prevbookmark,
   cmd_bookmarkLast,

   cmd_breakpointFirst,
   cmd_conditionalbreakpoint,
   cmd_breakpoint,
   cmd_breakpoint_remove,
   cmd_nextbreakpoint,
   cmd_prevbreakpoint,
   cmd_breakpointLast,

   cmd_tracedelay,

   cmd_logFirst,
   cmd_logclear,
   cmd_logsave,
   cmd_logverbose,
   cmd_logdebug,
   cmd_logLast,

   cmd_command,
   cmd_commandtext,
   
   cmd_variablesFirst,
   cmd_variablesgetall,
   cmd_variablesgetserver,
   cmd_variablesgetenv,
   cmd_variablesgetsession,
   cmd_variablesgetcookie,
   cmd_variablesgetthis,
   cmd_variablesgetglobals,
   cmd_variablesgetget,
   cmd_variablesgetrequest,
   cmd_variablesgetfiles,
   cmd_variablesgetpost,
   cmd_variablesgetspecified,
   cmd_variablesgetexpression,
   cmd_variablesLast,

   cmd_watchFirst,
   cmd_watchgetserver,
   cmd_watchgetenv,
   cmd_watchgetsession,
   cmd_watchgetcookie,
   cmd_watchgetthis,
   cmd_watchgetglobals,
   cmd_watchgetget,
   cmd_watchgetrequest,
   cmd_watchgetfiles,
   cmd_watchgetpost,
   cmd_watchgetspecified,
   cmd_watchgetexpression,
   cmd_watchLast,

   cmd_Last,

   sck_Server,

   grd_First,
   grd_backtracelist,
   grd_breakpointlist,
   grd_Last
   
};

#endif
