/*
 * Copyright (C) 2004 by Linus McCabe
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include "frmSettings.h"
#include  <wx/config.h>
#include "appGubed.h"

BEGIN_EVENT_TABLE(frmSettings, wxDialog)
EVT_BUTTON(cmd_settingok, frmSettings::OnCommand)
EVT_BUTTON(cmd_settingcancel, frmSettings::OnCommand)
EVT_CHECKBOX(cmd_settingsproxy, frmSettings::OnCommand)
EVT_CLOSE(frmSettings::OnClose)
END_EVENT_TABLE()

frmSettings::frmSettings(const wxString& title, const wxPoint& pos, const wxSize& size)
    : wxDialog((wxFrame *)NULL, -1, title, pos, size, wxRESIZE_BORDER | wxDEFAULT_DIALOG_STYLE, _T("SettingsDialog") )
{

  // Set the icon
  //SetIcon(wxIcon(wxICON(pause)));

  // Set up controls
  SetupControls();

  // Restore sizes
  wxConfigBase *cfgGubed = wxConfigBase::Get();
  SetSize(cfgGubed->Read(GetName() + wxT("/WindowLeft"), 0l)
        , cfgGubed->Read(GetName() + wxT("/WindowTop"), 0l)
        , cfgGubed->Read(GetName() + wxT("/WindowWidth"), 640l)
        , cfgGubed->Read(GetName() + wxT("/WindowHeight"), 300l));

}

void frmSettings::OnCommand(wxCommandEvent& event)
{
  wxConfigBase *cfgGubed = wxConfigBase::Get();
  long ErrorMask = 0;
  int intCnt;

  switch(event.GetId())
  {
    case cmd_settingok:

      cfgGubed->Write(wxT("Toolbar"), choToolbar->GetSelection());
      cfgGubed->Write(wxT("TabsLeft"), choTabs->GetSelection());
      cfgGubed->Write(wxT("Server"), chkProxy->GetValue() ? txtServer->GetValue() : wxString(wxT("")));
      cfgGubed->Write(wxT("Port"), txtPort->GetValue());
      cfgGubed->Write(wxT("UseProxy"), (long)chkProxy->GetValue());


      for(intCnt = 0; intCnt < 11; intCnt++)
        ErrorMask |= chkErrorMask[intCnt]->GetValue() ? (1 << intCnt ) : 0;

      cfgGubed->Write("ErrorMask", wxString::Format("%ld", ErrorMask));
      cfgGubed->Flush();
      EndModal(1);
      break;

    case cmd_settingcancel:
      EndModal(0);
      break;

    case cmd_settingsproxy:
      txtServer->Enable(chkProxy->GetValue());
      break;
  }

}

void frmSettings::OnClose(wxCloseEvent& event)
{
  EndModal(0);
}

void frmSettings::SetupControls()
{

  int intToolbar = 0;
  long ErrorMask;
  int intCnt;
  wxString strServer, strPort;

  wxConfigBase *cfgGubed = wxConfigBase::Get();

  wxBoxSizer * windowsizer = new wxBoxSizer(wxVERTICAL);
  wxNotebook * notebook = new wxNotebook(this, -1);
  windowsizer->Add(notebook, 1, wxEXPAND | wxALL, 4);

  // Connection settings
  wxPanel *panel = new wxPanel(notebook);
  wxGridSizer *gsWindow = new wxFlexGridSizer(2, 4, 25);

  if(wxGetApp().isConnected())
  {
    gsWindow->Add(new wxStaticText(panel, -1, _(""), wxDefaultPosition, wxDefaultSize), 1, wxEXPAND);
    gsWindow->Add(new wxStaticText(panel, -1, _("Unable to change connection settings while connected"), wxDefaultPosition, wxDefaultSize), 1, wxEXPAND);
  }

  wxStaticText *stLabel = new wxStaticText(panel, -1, _("Use proxy"), wxDefaultPosition, wxDefaultSize);
  chkProxy = new wxCheckBox(panel, cmd_settingsproxy, wxT(""), wxDefaultPosition, wxDefaultSize);
  gsWindow->Add(stLabel, 1, wxEXPAND);
  gsWindow->Add(chkProxy, 1, wxEXPAND);

  stLabel = new wxStaticText(panel, -1, _("Proxy server"), wxDefaultPosition, wxDefaultSize);
  txtServer = new wxTextCtrl(panel, -1, "", wxDefaultPosition, wxDefaultSize);
  gsWindow->Add(stLabel, 1, wxEXPAND);
  gsWindow->Add(txtServer, 1, wxEXPAND);


  stLabel = new wxStaticText(panel, -1, _("Port"), wxDefaultPosition, wxDefaultSize);
  txtPort = new wxTextCtrl(panel, -1, "", wxDefaultPosition, wxDefaultSize);
  gsWindow->Add(stLabel, 1, wxEXPAND);
  gsWindow->Add(txtPort, 1, wxEXPAND);

  panel->SetSizer(gsWindow);
  gsWindow->SetSizeHints(panel);
  notebook->AddPage(panel, _("Connection"), true);

  // If we have a connection established, you can't change connection settings
  panel->Enable(!wxGetApp().isConnected());

  // Error handling settings
  panel = new wxPanel(notebook);
  gsWindow = new wxFlexGridSizer(2, 4, 25);

  stLabel = new wxStaticText(panel, -1, "Break on", wxDefaultPosition, wxDefaultSize);
  wxBoxSizer *bsErr = new wxBoxSizer(wxVERTICAL);

  const char *pchChkLbl[] = {"E_ERROR", "E_WARNING", "E_PARSE", "E_NOTICE", "E_CORE_ERROR", "E_CORE_WARNING", "E_COMPILE_ERROR", "E_COMPILE_WARNING", "E_USER_ERROR", "E_USER_WARNING", "E_USER_NOTICE"};
  cfgGubed->Read("ErrorMask",  wxString::Format(wxT("%d"), (PHP_ERROR_E_ALL & ~PHP_ERROR_E_NOTICE))).ToLong(&ErrorMask);

  for(intCnt = 0; intCnt < 11; intCnt++)
  {
    chkErrorMask[intCnt] = new wxCheckBox(panel, -1, pchChkLbl[intCnt], wxDefaultPosition, wxDefaultSize);
    chkErrorMask[intCnt]->Enable(GBD_SETTINGS_ERRORMASK_INTERCEPTABLE & (1 << intCnt ));
    chkErrorMask[intCnt]->SetValue(ErrorMask & (1 << intCnt ));
    bsErr->Add(chkErrorMask[intCnt], 1, wxEXPAND);
  }

  gsWindow->Add(stLabel, 1, wxEXPAND);
  gsWindow->Add(bsErr, 1, wxEXPAND);

  panel->SetSizer(gsWindow);
  gsWindow->SetSizeHints(panel);
  notebook->AddPage(panel, _("Debug behaviour"), false);


  // Gui Toolbar setting
  wxString strChoices[] = {_("No toolbar"), _("Images only"), _("Text Only"), _("Images and text")};
  wxString strTabChoices[] = {_("Top"), _("Left")};

  panel = new wxPanel(notebook);
  gsWindow = new wxFlexGridSizer(2, 4, 25);

  stLabel = new wxStaticText(panel, -1, _("Toolbar"), wxDefaultPosition, wxDefaultSize);
  choToolbar = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 4, strChoices);
  gsWindow->Add(stLabel, 1, wxEXPAND );
  gsWindow->Add(choToolbar, 1, wxEXPAND);

  stLabel = new wxStaticText(panel, -1, _("Tabs (requires restart)"), wxDefaultPosition, wxDefaultSize);
  choTabs = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 2, strTabChoices);
  gsWindow->Add(stLabel, 1, wxEXPAND );
  gsWindow->Add(choTabs, 1, wxEXPAND);


  panel->SetSizer(gsWindow);
  gsWindow->SetSizeHints(panel);
  notebook->AddPage(panel, _("GUI"), false);


  // Ok and Cancel buttons
  wxBoxSizer * buttonsizer = new wxBoxSizer(wxHORIZONTAL);
  buttonsizer->Add(1, 1, 1, wxEXPAND);
  buttonsizer->Add(new wxButton(this, cmd_settingok, _("OK"), wxDefaultPosition, wxDefaultSize), 0, wxALL, 4);
  buttonsizer->Add(new wxButton(this, cmd_settingcancel, _("Cancel"), wxDefaultPosition, wxDefaultSize), 0, wxALL, 4);
  windowsizer->Add(buttonsizer, 0, wxEXPAND);

  // Alter controls from settings
  intToolbar = cfgGubed->Read(wxT("Toolbar"), 3l);
  strServer =  cfgGubed->Read(wxT("Server"), wxT("localhost"));
  strPort =  cfgGubed->Read(wxT("Port"), wxT("8016"));

  chkProxy->SetValue(cfgGubed->Read(wxT("UseProxy"), 0l));
  txtServer->Enable(chkProxy->GetValue());

  txtServer->SetValue(strServer);
  txtPort->SetValue(strPort);
  choToolbar->SetSelection(intToolbar);
  choTabs->SetSelection(cfgGubed->Read(wxT("TabsLeft"), 0l));

  SetSizer(windowsizer);
//  windowsizer->SetSizeHints(this);



}

frmSettings::~frmSettings()
{
  wxConfigBase *cfgGubed = wxConfigBase::Get();

  wxRect rctPos = this->GetRect();
  cfgGubed->Write(GetName() + wxT("/WindowLeft"), rctPos.GetLeft());
  cfgGubed->Write(GetName() + wxT("/WindowTop"), rctPos.GetTop());
  cfgGubed->Write(GetName() + wxT("/WindowWidth"), rctPos.GetWidth());
  cfgGubed->Write(GetName() + wxT("/WindowHeight"), rctPos.GetHeight());
  cfgGubed->Flush();

}
