/*
 * Copyright (C) 2004 by Linus McCabe
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#ifndef FRMSETTINGS_H
#define FRMSETTINGS_H

#include "wx/wx.h"

class frmSettings: public wxDialog
{
  private:

    void OnCommand(wxCommandEvent&);
    void SetupControls(void);
    void OnClose(wxCloseEvent&);

    wxChoice *choToolbar;
    wxChoice *choTabs;
    wxTextCtrl *txtServer;
    wxTextCtrl *txtPort;
    wxCheckBox *chkErrorMask[11];
    wxCheckBox *chkProxy;

  public:
    frmSettings(const wxString& title, const wxPoint& pos, const wxSize& size);
    ~frmSettings(void);

    DECLARE_EVENT_TABLE()
};

enum {
  cmd_settingfirst,
  cmd_settingok,
  cmd_settingcancel,
  cmd_settingsproxy,
  cmd_settinglast
};



/*
	PHP Error/Warning codes. Not all of these are interceptable, but they are included for completeness
	Uninterceptable: E_ERROR, E_PARSE, E_CORE_ERROR, E_CORE_WARNING, E_COMPILE_ERROR and E_COMPILE_WARNING.
	Interceptable:   E_WARNING, E_NOTICE, E_USER_ERROR, E_USER_WARNING and E_USER_NOTICE
*/
//Fatal run-time errors. These indicate errors that can not be recovered from, such as a memory allocation problem. Execution of the script is halted.
#define PHP_ERROR_E_ERROR 1
#define GBD_SETTINGS_CHECKBOX_E_ERROR 0

// Run-time warnings (non-fatal errors). Execution of the script is not halted.
#define PHP_ERROR_E_WARNING 2
#define GBD_SETTINGS_CHECKBOX_E_WARNING 1

//Compile-time parse errors. Parse errors should only be generated by the parser.
#define PHP_ERROR_E_PARSE 4
#define GBD_SETTINGS_CHECKBOX_E_PARSE 2

// Run-time notices. Indicate that the script encountered something that could indicate an error, but could also happen in the normal course of running a script.
#define PHP_ERROR_E_NOTICE 8
#define GBD_SETTINGS_CHECKBOX_E_NOTICE 3

// Fatal errors that occur during PHP's initial startup. This is like an E_ERROR, except it is generated by the core of PHP.
#define PHP_ERROR_E_CORE_ERROR 16
#define GBD_SETTINGS_CHECKBOX_E_CORE_ERROR 4

// Warnings (non-fatal errors) that occur during PHP's initial startup. This is like an E_WARNING, except it is generated by the core of PHP.
#define PHP_ERROR_E_CORE_WARNING 32
#define GBD_SETTINGS_CHECKBOX_E_CORE_WARNING 5

// Fatal compile-time errors. This is like an E_ERROR, except it is generated by the Zend Scripting Engine.
#define PHP_ERROR_E_COMPILE_ERROR 64
#define GBD_SETTINGS_CHECKBOX_E_COMPILE_ERROR 6

// Compile-time warnings (non-fatal errors). This is like an E_WARNING, except it is generated by the Zend Scripting Engine.
#define PHP_ERROR_E_COMPILE_WARNING 128
#define GBD_SETTINGS_CHECKBOX_E_COMPILE_WARNING 7

//User-generated error message. This is like an E_ERROR, except it is generated in PHP code by using the PHP function trigger_error().
#define PHP_ERROR_E_USER_ERROR 256
#define GBD_SETTINGS_CHECKBOX_E_USER_ERROR 8

// User-generated warning message. This is like an E_WARNING, except it is generated in PHP code by using the PHP function trigger_error().
#define PHP_ERROR_E_USER_WARNING 512
#define GBD_SETTINGS_CHECKBOX_E_USER_WARNING 9

#define PHP_ERROR_E_USER_NOTICE	1024
#define GBD_SETTINGS_CHECKBOX_E_USER_NOTICE	10

#define PHP_ERROR_E_ALL 2047
#define GBD_SETTINGS_ERRORMASK_INTERCEPTABLE (PHP_ERROR_E_WARNING | PHP_ERROR_E_NOTICE | PHP_ERROR_E_USER_ERROR | PHP_ERROR_E_USER_WARNING | PHP_ERROR_E_USER_NOTICE)

#endif
