/*
 * Copyright (C) 2004 by Linus McCabe
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include "nbSources.h"
#include "appGubed.h"

BEGIN_EVENT_TABLE(nbSources, wxNotebook)
//   EVT_COMMAND_WORDCLICK(-1, nbSources::OnAbout)
END_EVENT_TABLE()

nbSources::nbSources(wxWindow* parent, long style)
    : wxNotebook((wxWindow *)parent, -1, wxDefaultPosition, wxDefaultSize, style)
{
  SetupControls();
}

void nbSources::SetupControls()
{
}
void nbSources::ToggleBookmark(long lngLineNum)
{
  if(GetSelection() >= 0)
  {
    panSourceView *psvShown = (panSourceView *)GetPage(GetSelection());
    if(psvShown)
      psvShown->ToggleBookmark(lngLineNum);
  }
}

void nbSources::ToggleBreakpoint(long lngLineNum)
{
  if(GetSelection() >= 0)
  {
    panSourceView *psvShown = (panSourceView *)GetPage(GetSelection());
    if(psvShown)
      psvShown->ToggleBreakpoint(lngLineNum);
  }
}

void nbSources::SetBreakpoint(const wxString & strSource, long lngLineNum)
{
  if(hmsSources.find(strSource) != hmsSources.end())
  {
    hmsSources[strSource]->SetBreakpoint(lngLineNum, false);
  }
}

void nbSources::ClearBreakpoint(const wxString & strSource, long lngLineNum)
{
  if(hmsSources.find(strSource) != hmsSources.end())
  {
    hmsSources[strSource]->ClearBreakpoint(lngLineNum, false);
  }
}

// void nbSources::ParsedOk(const wxString& strSource)
// {
// 
//   if(hmsSources.find(strSource) != hmsSources.end())
//     if(GetPageText(hmsSources[strSource]->GetPage()).Right(2) == " *")
//       SetPageText(hmsSources[strSource]->GetPage(), GetPageText(hmsSources[strSource]->GetPage()).Left(GetPageText(hmsSources[strSource]->GetPage()).Length() - 2));
//   return;
// 
// }
// 
// void nbSources::UnParsed(const wxString& strSource)
// {
// 
//   if(hmsSources.find(strSource) != hmsSources.end())
//     if(GetPageText(hmsSources[strSource]->GetPage()).Right(2) != " *")
//       SetPageText(hmsSources[strSource]->GetPage(), GetPageText(hmsSources[strSource]->GetPage()) + " *");
//   return;
// 
// }

void nbSources::gotoFileAndLine(const wxString & filename, long line)
{
  if(hmsSources.find(filename) != hmsSources.end())
  {
    hmsSources[filename]->ShowSourceLine(line);
    SetSelection(hmsSources[filename]->GetPage());
  }

  this->Refresh();
}


void nbSources::SetActiveSourceLine(const wxString & strSource, long lngLineNum)
{

  if(strSource != strActiveSource && strActiveSource != "" && hmsSources.find(strActiveSource) != hmsSources.end())
    hmsSources[strActiveSource]->SetActiveSourceLine(-1);

  if(hmsSources.find(strSource) != hmsSources.end())
  {
    strActiveSource = strSource;
    hmsSources[strSource]->SetActiveSourceLine(lngLineNum);
    if(GetSelection() != hmsSources[strSource]->GetPage())
      SetSelection(hmsSources[strSource]->GetPage());

  }
  else
  {
    wxGetApp().SendCommand(wxT("reinitialize"));
    strActiveSource = "";
  }

  this->Refresh();
}

void nbSources::ShowActiveSourceLine()
{

  if(hmsSources.find(strActiveSource) != hmsSources.end())
  {
    hmsSources[strActiveSource]->ShowActiveSourceLine();
    SetSelection(hmsSources[strActiveSource]->GetPage());

  }

}

void nbSources::SetSource(const wxString &File, const wxString &Source)
{
  hmsSources[File]->SetSource(Source);

  wxArrayInt bps = wxGetApp().getFileBreakpoints(File);
  for(unsigned int cnt = 0; cnt < bps.Count(); cnt++)
  {
    hmsSources[File]->SetBreakpoint(bps[cnt], true);
  }
}


bool nbSources::AddSource(const wxString& strSource, const wxString& strIdentifier)
{
  panSourceView* psvAdd;

  if(hmsSources.find(strSource) == hmsSources.end())
  {
    if(!(psvAdd = new panSourceView(this, ::wxNewId(), wxDefaultPosition, wxDefaultSize, strSource)))
      return false;

    wxString name = strSource;
    name.Replace(_T("\\"), _T("/"), true);
    name = name.AfterLast('/');
    AddPage(psvAdd, name);
    hmsSources[strSource] = psvAdd;
//    psvAdd->SetPage(GetPageCount() - 1);

  }

  // Update menues and toolbar
  wxGetApp().GetGubedWindow()->EnableCommand(cmd_closefile, true);

  return hmsSources[strSource]->SetIdentifier(strIdentifier);
}


void nbSources::CloseSource()
{
//   int page = GetSelection();
//   int pagec = GetPageCount();

  if(GetSelection() >= 0 && GetSelection() < GetPageCount())
  {
    panSourceView* current = (panSourceView*) GetPage(GetSelection());
    if(current)// && current->GetFilename() != strActiveSource)
    {
      hmsSources.erase(current->GetFilename());
      DeletePage(GetSelection());

      if(GetSelection() > GetPageCount())
        SetSelection(0);

    }
  }
  wxGetApp().GetGubedWindow()->EnableCommand(cmd_closefile, GetPageCount());
}
