/*
 * Copyright (C) 2004 by Linus McCabe
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#ifdef __WIN32__
	#include "../configmingw.h"
#else
	#include "../config.h"
#endif
#ifndef HAVE_STC

#include "panSourceView.h"
#include "appGubed.h"
#include "frmConditionalBreakpoint.h"
#include <wx/regex.h>
#include  <wx/tooltip.h>

BEGIN_EVENT_TABLE(panSourceView, wxPanel)
EVT_GRID_CELL_LEFT_CLICK(panSourceView::OnLeftClick)
EVT_GRID_CELL_RIGHT_CLICK(panSourceView::OnRightClick)
EVT_GRID_RANGE_SELECT(panSourceView::OnRangeSelect)
//EVT_GRID_SELECT_CELL(panSourceView::OnCellSelect)

EVT_MENU(cmd_variable_inspect, panSourceView::OnView)
EVT_MENU(cmd_variable_watch, panSourceView::OnWatch)
EVT_MENU(cmd_variable_set, panSourceView::OnSet)
EVT_MENU(cmd_variable_break, panSourceView::OnCondition)

END_EVENT_TABLE()

panSourceView::panSourceView(wxWindow* parent, wxWindowID id = -1,
                             const wxPoint& pos = wxDefaultPosition,
                             const wxSize& size = wxDefaultSize,
                             const wxString& strFilename = "")
    : wxPanel((wxWindow *)parent, id, pos, size)
{
  lngCurrentSourceLine = -1;
  SetupControls();
  txtSourceName->SetValue(strFilename);
  strSourcename = strFilename;

  int intW, intH;
  wxFont fontGrid = grdSourceView->GetDefaultCellFont();
  grdSourceView->GetTextExtent("X", &intW, &intH,  &intH,  &intH, &fontGrid);
  intFontWidth = intW;

}

void panSourceView::SetupControls()
{

  wxBoxSizer *bsWindow = new wxBoxSizer(wxVERTICAL);

  grdSourceView = new wxGrid(this, -1, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER /*| wxWANTS_CHARS*/);
  //
  grdSourceView->CreateGrid(1, 3);
  grdSourceView->SetColSize(0, 20);
  grdSourceView->SetColSize(1, 20);
  grdSourceView->SetColSize(2, 400);

  grdSourceView->SetDefaultCellFont(wxFont(10, wxMODERN, wxNORMAL, wxNORMAL, false));
  grdSourceView->EnableGridLines(false); //(wxColour(255, 255, 255));
  grdSourceView->EnableEditing(false);
  grdSourceView->DisableDragRowSize();
  grdSourceView->SetSelectionMode(wxGrid::wxGridSelectRows);
  grdSourceView->SetSelectionBackground(wxColour(255, 255, 255));
  grdSourceView->SetSelectionForeground(wxColour(0, 0, 0));
  grdSourceView->SetColLabelSize(0);
  grdSourceView->SetRowLabelSize(50);
  grdSourceView->SetRowLabelAlignment(wxALIGN_RIGHT, wxALIGN_TOP);

  wxBoxSizer *bsInfo = new wxBoxSizer(wxHORIZONTAL);
  txtSourceName = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
  txtCurrentline = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
  txtTotallines = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
  bsInfo->Add(txtSourceName, 5, wxEXPAND);
  bsInfo->Add(txtCurrentline, 2, wxEXPAND);
  bsInfo->Add(txtTotallines, 1, wxEXPAND);

  bsWindow->Add(bsInfo, 0, wxEXPAND);
  bsWindow->Add(grdSourceView, 1, wxEXPAND);

  SetSizer(bsWindow);

}

/*void panSourceView::SetPage(int intThisPage)
{
  intPage = intThisPage;
}
*/

int panSourceView::GetPage()
{
  wxNotebook* parent = (wxNotebook*)GetParent();
  for(int cnt = 0; cnt < parent->GetPageCount(); cnt++)
  {
    if(parent->GetPage(cnt) == this)
      return cnt;
  }

  return -1;
}

bool panSourceView::SetIdentifier(const wxString &strIdent)
{
  if(grdSourceView->GetNumberRows() < 2)
  {
    strIdentifier = strIdent;
    return false;
  }

  if(strIdentifier == strIdent)
    return true;

  strIdentifier = strIdent;
  return false;
}


void panSourceView::SetSource(wxString Source)
{
  wxString strSourceLine, strInfo;
  grdSourceView->BeginBatch();
  long lngLineNum = 0, pos;
  while(Source.Length() > 0)
  {
    pos = Source.Find(wxT("\n"));
    strSourceLine = Source.Left(pos);
    if(pos < 0)
      break;

    Source.Remove(0, pos);
    if(Source.Length() > 0 && Source.Left(1) == 13) // Handle CR+LF, Only LF and only CR
      Source.Remove(0, 1);
    if(Source.Length() > 0 && Source.Left(1) == 10)
      Source.Remove(0, 1);

    if(grdSourceView->GetNumberRows() <= lngLineNum)
      grdSourceView->AppendRows(1 + lngLineNum - grdSourceView->GetNumberRows());

    if(lngLineNum >= 0)
    {
      grdSourceView->SetCellValue(lngLineNum, 2, strSourceLine);

      grdSourceView->SetCellBackgroundColour(lngLineNum, 0, wxColour(200, 200, 200));
      grdSourceView->SetCellBackgroundColour(lngLineNum, 1, wxColour(200, 200, 200));
    }
    lngLineNum++;
  }


  grdSourceView->AutoSizeColumn(2);
  grdSourceView->EndBatch();

  strInfo.Printf("%d lines", grdSourceView->GetNumberRows());
  txtTotallines->SetValue(strInfo);

}


void panSourceView::SetActiveSourceLine(long lngLineNum)
{

  long lngPrevCurrentSourceLine = lngCurrentSourceLine;
  wxString strInfo;

  if(lngLineNum >= grdSourceView->GetNumberRows())
    lngLineNum = -1;

  lngCurrentSourceLine = lngLineNum;
  SetRowColour(lngPrevCurrentSourceLine);

  if(lngCurrentSourceLine >= 0)
  {
    SetRowColour(lngCurrentSourceLine);
    grdSourceView->MakeCellVisible(lngCurrentSourceLine , 0);
    strInfo.Printf(_("Currently at line %ld"), lngCurrentSourceLine + 1L);

  }
  else
    strInfo = "";

  txtCurrentline->SetValue(strInfo);

}

void panSourceView::ShowActiveSourceLine()
{
  grdSourceView->MakeCellVisible(lngCurrentSourceLine , 0);
  grdSourceView->ForceRefresh();

}
void panSourceView::ShowSourceLine(long line)
{
  grdSourceView->MakeCellVisible(line , 0);
  grdSourceView->ForceRefresh();

}

void panSourceView::SetBreakpoint(long lngLineNum, bool blInform)
{

  if(lngLineNum >= 0)
  {
    grdSourceView->SetCellValue(lngLineNum, 1, "*");

    if(blInform)
    {
      Breakpoint bp(Breakpoint::Line, strSourcename, lngLineNum);
      wxGetApp().setBreakpoint(bp);
    }
    
    SetRowColour(lngLineNum);
  }
}

void panSourceView::ClearBreakpoint(long lngLineNum, bool blInform)
{

  if(lngLineNum >= 0)
  {
    grdSourceView->SetCellValue(lngLineNum, 1, "");
    
    if(blInform)
    {
      Breakpoint bp(Breakpoint::Line, strSourcename, lngLineNum);
      wxGetApp().clearBreakpoint(bp);
    }
    
    SetRowColour(lngLineNum);
  }
}

void panSourceView::ToggleBreakpoint(long lngLineNum)
{

  if(lngLineNum == -1)
    lngLineNum = grdSourceView->GetGridCursorRow();

  if(lngLineNum >= 0)
  {
    if(grdSourceView->GetCellValue(lngLineNum, 1) == "")
      SetBreakpoint(lngLineNum, true);
    else
      ClearBreakpoint(lngLineNum, true);
  }

  return;
}
void panSourceView::ToggleBookmark(long lngLineNum)
{

  if(lngLineNum == -1)
    lngLineNum = grdSourceView->GetGridCursorRow();

  if(lngLineNum >= 0)
  {
    if(grdSourceView->GetCellValue(lngLineNum, 0) == "")
    {
      grdSourceView->SetCellValue(lngLineNum, 0, "*");
      wxGetApp().setBookmark(strSourcename, lngLineNum);
    }
    else
    {
      grdSourceView->SetCellValue(lngLineNum, 0, "");
      wxGetApp().clearBookmark(strSourcename, lngLineNum);
    }
    SetRowColour(lngLineNum);
  }

  return;
}

void panSourceView::SetRowColour(long lngLineNum)
{

  wxColor colBookmark;
  wxColor colBreakpoint;
  wxColor colSource;

  bool blMark = false;

  if(lngLineNum < 0)
    return;

  if(grdSourceView->GetCellValue(lngLineNum, 0) != "")
  {
    colBookmark = wxColour(80, 190, 255);
    blMark = true;
  }
  else
    colBookmark = wxColour(200, 200, 200);

  if(grdSourceView->GetCellValue(lngLineNum, 1) != "")
  {
    colBreakpoint = wxColour(255, 70, 70);
    blMark = true;
  }
  else
    colBreakpoint = colBookmark;

  if(lngCurrentSourceLine == lngLineNum)
    colSource = wxColour(255, 255, 120);
  else
    colSource = blMark ? colBreakpoint : wxColour(255, 255, 255);

  grdSourceView->SetCellBackgroundColour(lngLineNum, 0, colBookmark);
  grdSourceView->SetCellBackgroundColour(lngLineNum, 1, colBreakpoint);
  grdSourceView->SetCellBackgroundColour(lngLineNum, 2, colSource);


}

bool panSourceView::CheckForWord(int intRow, int intLeftPos, wxString &strWord)
{

  wxString strLine;

#ifndef __WIN32__
  // According to php.net:
  wxRegEx reVariable("\\$?[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*(\\[.*?\\])*");
#else
  // But it gives error on win32...
  wxRegEx reVariable("\\$?[a-zA-Z_][a-zA-Z0-9_]*"); //(\\[.*?\\])*
#endif

  int intX, intUX;
  size_t stPosStart, stPosEnd;

  if(intFontWidth <= 0)
    return false;

  strLine = grdSourceView->GetCellValue(intRow, 2);

  grdSourceView->GetScrollPixelsPerUnit(&intUX, NULL);
  grdSourceView->GetViewStart(&intX, NULL);
  intX *= -intUX;
  intX += grdSourceView->GetRowLabelSize();
  intX += grdSourceView->GetColSize(0);
  intX += grdSourceView->GetColSize(1);

  intX = (intLeftPos - intX) / intFontWidth;

  if(intX > (int)strLine.size())
    return false;

  while(reVariable.Matches(strLine) && intX >= 0)
  {

    if(reVariable.GetMatch(&stPosStart, &stPosEnd, 0))
    {
      if((int)stPosStart < intX && (int)stPosEnd > intX)
      {
        strWord = strLine.substr(stPosStart, stPosEnd);
        //wxGetApp().GetGubedWindow()->SendCommand("getvariable\n" + strLine.substr(stPosStart, stPosEnd) + "\n");
        return true;
      }
    }
    strLine = strLine.substr(1);
    intX--;
  }

  return false;
}

// Events

void panSourceView::OnRangeSelect(wxGridRangeSelectEvent& event)
{
  if(grdSourceView->IsSelection())
    grdSourceView->ClearSelection();
  //grdSourceView->MoveCursorLeft(false);
  //grdSourceView->MoveCursorLeft(false);
  grdSourceView->MakeCellVisible(event.GetBottomRow(), 0);
  return;
}
/*
void panSourceView::OnCellSelect(wxGridEvent& event)
{
  if(grdSourceView->IsSelection())
    grdSourceView->ClearSelection();
  //grdSourceView->MoveCursorLeft(false);
  //grdSourceView->MoveCursorLeft(false);
  grdSourceView->MakeCellVisible(event.GetRow(), 0);

  return;
}*/

void panSourceView::OnLeftClick(wxGridEvent& event)
{

  wxString strWord;

  switch(event.GetCol())
  {
    case 0:
      ToggleBookmark(event.GetRow());
      return;

    case 1:
      ToggleBreakpoint(event.GetRow());
      return;

    case 2:
      if(CheckForWord(event.GetRow(), event.GetPosition().x, strWord))
      {
        //wxGetApp().SendCommand(wxT("getvariable"), strWord);
        wxGetApp().SendCommand(wxT("getvariable"), CompileHash("variable", strPopupWord.c_str(), 0));
        return;
      }
      break;
  }
  grdSourceView->SetGridCursor(event.GetRow(), 0);
  // Dont let clicks slip through
  // event.Skip();
  return;
}

void panSourceView::OnRightClick(wxGridEvent& event)
{
  wxString strWord;

  switch(event.GetCol())
  {

    case 2:
      if(CheckForWord(event.GetRow(), event.GetPosition().x, strWord))
      {
        wxMenu menu(strWord);
        menu.Append(cmd_variable_inspect, "Sho&w");
        menu.Append(cmd_variable_watch, "&Watch");
        menu.Append(cmd_variable_set, "&Set");
        menu.Append(cmd_variable_break, "&Break when...");

        strPopupWord = strWord;

        PopupMenu(&menu, event.GetPosition().x, event.GetPosition().y);


        return;
      }
      break;

  }
  event.Skip();
  return;
}


void panSourceView::OnWatch(wxCommandEvent& event)
{
  wxGetApp().AddWatch(strPopupWord);
}

void panSourceView::OnView(wxCommandEvent& event)
{
  //wxGetApp().SendCommand(wxT("getvariable"), strPopupWord);
  wxGetApp().SendCommand(wxT("getvariable"), CompileHash("variable", strPopupWord.c_str(), 0));
}

void panSourceView::OnSet(wxCommandEvent& event)
{

  wxTextEntryDialog *teValue =
    new wxTextEntryDialog(
      wxGetApp().GetGubedWindow(),
      _("Enter value for: ") + strPopupWord,
      _("Set variable contents"),
      wxT(""));

  if(teValue->ShowModal() ==  wxID_OK)
    //wxGetApp().SendCommand(wxT("setvariable"), strPopupWord + "=" + teValue->GetValue());
      wxGetApp().SendCommand(wxT("setvariable"), CompileHash("variable", strPopupWord.c_str(), "value", teValue->GetValue().c_str(),  0));
}

void panSourceView::OnCondition(wxCommandEvent& event)
{
  frmConditionalBreakpoint *condition;

  condition = new frmConditionalBreakpoint(wxGetApp().GetGubedWindow());
  condition->SetFilename(strSourcename);
  condition->SetExpression(strPopupWord);

  if(condition->ShowModal() ==  wxID_OK)
  {
    if(condition->GetExpression() != _T("")) 
    {
      Breakpoint bp(condition->GetType(), 
                    condition->GetExpression(),
                    condition->GetFilename(),
                    condition->GetClass(),
                    condition->GetFunction()
                    );
      wxGetApp().setBreakpoint(bp);
    }
  }
  delete condition;
}


#endif
