/*
 * Copyright (C) 2004 by Linus McCabe
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifdef __WIN32__
	#include "../configmingw.h"
#else
	#include "../config.h"
#endif
#ifdef HAVE_STC

#include "panSourceViewSTC.h"
#include "appGubed.h"
#include "frmConditionalBreakpoint.h"
#include "Breakpoint.h"

#include <wx/regex.h>
#include <wx/tooltip.h>
#include <wx/stc/stc.h>
#include <wx/notebook.h>

// Bitmaps
#define USE_XPM_BITMAPS 1
#include "bitmaps/run.xpm"
#include "bitmaps/bookmark.xpm"
#include "bitmaps/breakpoint.xpm"


BEGIN_EVENT_TABLE(panSourceView, wxPanel)
  EVT_STC_MARGINCLICK(-1, panSourceView::OnMarginClick)
  EVT_STC_HOTSPOT_CLICK(-1, panSourceView::OnLeftClick)
  EVT_RIGHT_UP(panSourceView::OnRightClick)
  EVT_MENU(cmd_variable_inspect, panSourceView::OnView)
  EVT_MENU(cmd_variable_watch, panSourceView::OnWatch)
  EVT_MENU(cmd_variable_set, panSourceView::OnSet)
  EVT_MENU(cmd_variable_break, panSourceView::OnCondition)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE (gbdStyledTextCtrl, wxStyledTextCtrl)
  EVT_RIGHT_UP(gbdStyledTextCtrl::OnMouseRightUp)
END_EVENT_TABLE()

gbdStyledTextCtrl::gbdStyledTextCtrl(wxWindow *parent,
                                     wxWindowID id,
                                     const wxPoint &pos,
                                     const wxSize &size,
                                     long style)
  : wxStyledTextCtrl (parent, id, pos, size, style)
{
}

gbdStyledTextCtrl::~gbdStyledTextCtrl()
{
}

void gbdStyledTextCtrl::OnMouseRightUp(wxMouseEvent& evt)
{
  GetParent()->ProcessEvent(evt);
};

panSourceView::panSourceView(wxWindow* parent, wxWindowID id = -1,
                             const wxPoint& pos = wxDefaultPosition,
                             const wxSize& size = wxDefaultSize,
                             const wxString& strFilename = "")
    : wxPanel((wxWindow *)parent, id, pos, size)
{
  lngCurrentSourceLine = -1;
  SetupControls();
  txtSourceName->SetValue(strFilename);
  strSourcename = strFilename;
}

void panSourceView::SetupControls()
{

  wxBoxSizer *bsWindow = new wxBoxSizer(wxVERTICAL);

  m_source = new gbdStyledTextCtrl(this);
  m_source->SetMarginType (MarginLinenum, wxSTC_MARGIN_NUMBER);
  m_source->SetMarginWidth(MarginLinenum, m_source->TextWidth (wxSTC_STYLE_LINENUMBER, _T("_999")));
  m_source->SetReadOnly(true);
  m_source->MarkerDefineBitmap(MarkerActiveLine, wxBitmap(run_xpm));
  m_source->MarkerSetForeground(MarkerActiveLine,  wxColour(255, 255, 210));
  m_source->MarkerDefineBitmap(MarkerBookmark, wxBitmap(bookmark_xpm));
  m_source->MarkerSetForeground(MarkerBookmark, wxColour(80, 190, 255));
  m_source->MarkerDefineBitmap(MarkerBreakpoint, wxBitmap(breakpoint_xpm));
  m_source->MarkerSetBackground(MarkerBreakpoint,  wxColour(255, 70, 70));

  m_source->SetMarginWidth(MarginBreakpoint, 16);
  m_source->SetMarginSensitive(MarginBreakpoint, true);
  m_source->SetMarginSensitive(MarginActiveLine, true);
  m_source->SetMarginMask(MarginBreakpoint, 1 << MarkerBreakpoint );
  m_source->SetMarginMask(MarginActiveLine, (1 << MarkerActiveLine) + (1 << MarkerBookmark));

  m_source->UsePopUp(false);

  m_source->SetLexer(wxSTC_LEX_HTML);
  m_source->SetStyleBits(7);

//  m_source->SetKeyWords(0, "HTML A a HREF href title TITLE");
  m_source->SetKeyWords(4, "or xor __FILE__ exception php_user_filter "
                            "__LINE__ array as break case "
                            "cfunction class const continue declare "
                            "default die do echo else "
                            "elseif empty enddeclare endfor endforeach "
                            "endif endswitch endwhile eval exit "
                            "extends for foreach function global "
                            "if include include_once isset list "
                            "new old_function print require require_once "
                            "return static switch unset use "
                            "var while __FUNCTION__ __CLASS__ __METHOD__ ");

  m_source->StyleSetSpec(wxSTC_H_TAG, wxT("fore:#aa4444"));
  m_source->StyleSetSpec(wxSTC_H_TAGUNKNOWN, wxT("fore:#aa4444"));
  m_source->StyleSetSpec(wxSTC_H_ATTRIBUTE, wxT("fore:#995555"));
  m_source->StyleSetSpec(wxSTC_H_ATTRIBUTEUNKNOWN, wxT("fore:#995555"));
  m_source->StyleSetSpec(wxSTC_H_COMMENT, wxT("fore:#999999"));

  m_source->StyleSetSpec(wxSTC_HPHP_DEFAULT, wxT("fore:#000000"));
  m_source->StyleSetSpec(wxSTC_HPHP_HSTRING, wxT("fore:#ff5555"));
  m_source->StyleSetSpec(wxSTC_HPHP_SIMPLESTRING, wxT("fore:#ff5555"));
  m_source->StyleSetSpec(wxSTC_HPHP_WORD, wxT("fore:#111188"));
  m_source->StyleSetSpec(wxSTC_HPHP_NUMBER, wxT("fore:#3333dd"));
  m_source->StyleSetSpec(wxSTC_HPHP_VARIABLE, wxT("fore:#3333dd"));
  m_source->StyleSetSpec(wxSTC_HPHP_COMMENT, wxT("fore:#999999"));
  m_source->StyleSetSpec(wxSTC_HPHP_COMMENTLINE, wxT("fore:#999999"));
  m_source->StyleSetSpec(wxSTC_HPHP_HSTRING_VARIABLE, wxT("fore:#3333dd"));
  m_source->StyleSetSpec(wxSTC_HPHP_OPERATOR, wxT("fore:#000000"));
  m_source->StyleSetHotSpot(wxSTC_HPHP_VARIABLE, true);

  wxBoxSizer *bsInfo = new wxBoxSizer(wxHORIZONTAL);
  txtSourceName = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
  txtCurrentline = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
  txtTotallines = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
  bsInfo->Add(txtSourceName, 5, wxEXPAND);
  bsInfo->Add(txtCurrentline, 2, wxEXPAND);
  bsInfo->Add(txtTotallines, 1, wxEXPAND);

  bsWindow->Add(bsInfo, 0, wxEXPAND);
  bsWindow->Add(m_source, 1, wxEXPAND);

  SetSizer(bsWindow);

}

/*void panSourceView::SetPage(int intThisPage)
{
  intPage = intThisPage;
}*/

int panSourceView::GetPage()
{
  wxNotebook* parent = (wxNotebook*)GetParent();
  for(int cnt = 0; cnt < parent->GetPageCount(); cnt++)
  {
    if(parent->GetPage(cnt) == this)
      return cnt;
  }

  return -1;
}

bool panSourceView::SetIdentifier(const wxString &strIdent)
{

  if(m_source->GetLineCount() < 2)
  {
    strIdentifier = strIdent;
    return false;
  }

  if(strIdentifier == strIdent)
    return true;

  strIdentifier = strIdent;
  return false;
}

void panSourceView::SetSource(wxString Source)
{
  m_source->SetReadOnly(false);
  m_source->SetText(Source);
  m_source->SetReadOnly(true);

  wxString strInfo;
  strInfo.Printf(_("%d lines"), m_source->GetLineCount());
  txtTotallines->SetValue(strInfo);
}


void panSourceView::SetActiveSourceLine(long lngLineNum)
{

  long lngPrevCurrentSourceLine = lngCurrentSourceLine;
  wxString strInfo;

  lngCurrentSourceLine = lngLineNum;
  m_source->MarkerDelete(lngPrevCurrentSourceLine, MarkerActiveLine);
  //SetRowColour(lngPrevCurrentSourceLine);

  if(lngCurrentSourceLine >= 0)
  {
    m_source->MarkerAdd(lngCurrentSourceLine, MarkerActiveLine);
    //SetRowColour(lngCurrentSourceLine);
    ShowActiveSourceLine();
    strInfo.Printf(_("Currently at line %ld"), lngCurrentSourceLine + 1L);
  }
  else
    strInfo = "";

  txtCurrentline->SetValue(strInfo);

}

void panSourceView::ShowActiveSourceLine()
{
  if(m_source->GetCurrentLine() != lngCurrentSourceLine)
    m_source->GotoLine(lngCurrentSourceLine);
}
void panSourceView::ShowSourceLine(long line)
{
  m_source->GotoLine(line);
}

void panSourceView::SetBreakpoint(long lngLineNum, bool blInform)
{
  if(lngLineNum >= 0)
  {
    
    m_source->MarkerAdd(lngLineNum, MarkerBreakpoint);
    if(blInform)
    {
      Breakpoint bp(Breakpoint::Line, strSourcename, lngLineNum);
      wxGetApp().setBreakpoint(bp);
    }
  }
}

void panSourceView::ClearBreakpoint(long lngLineNum, bool blInform)
{
  if(lngLineNum >= 0)
  {
    
    m_source->MarkerDelete(lngLineNum, MarkerBreakpoint);
    if(blInform)
    {
      Breakpoint bp(Breakpoint::Line, strSourcename, lngLineNum);
      wxGetApp().clearBreakpoint(bp);
    }
  }
}

void panSourceView::ToggleBreakpoint(long lngLineNum)
{

  if(lngLineNum == -1)
    lngLineNum = m_source->GetCurrentLine();

  if(lngLineNum >= 0)
  {
    if(!(m_source->MarkerGet(lngLineNum) & (1 << MarkerBreakpoint)))
      SetBreakpoint(lngLineNum, true);
    else
      ClearBreakpoint(lngLineNum, true);
  }

  return;
}
void panSourceView::ToggleBookmark(long lngLineNum)
{

  if(lngLineNum == -1)
    lngLineNum = m_source->GetCurrentLine();

  if(lngLineNum >= 0)
  {
    if(!(m_source->MarkerGet(lngLineNum) & (1 << MarkerBookmark)))
    {
      m_source->MarkerAdd(lngLineNum, MarkerBookmark);
      wxGetApp().setBookmark(strSourcename, lngLineNum);
    }
    else
    {
      m_source->MarkerDelete(lngLineNum, MarkerBookmark);
      wxGetApp().clearBookmark(strSourcename, lngLineNum);
    }
    //SetRowColour(lngLineNum);
  }

  return;
}

/*void panSourceView::SetRowColour(long lngLineNum)
{

  wxColor colBookmark;
  wxColor colBreakpoint;
  wxColor colSource;

  bool blMark = false;

  if(lngLineNum < 0)
    return;

}
*/
bool panSourceView::CheckForWord(const wxPoint& point, wxString &word)
{
  wxString chr;
  word = m_source->GetSelectedText();
  word = word.Left(word.Find(wxT("\n")));
  if(word == "")
  {
    int pos = m_source->PositionFromPoint(point);
    int startpos, endpos;

    startpos = m_source->WordStartPosition(pos, true);
    endpos = m_source->WordEndPosition(pos, true);
    word = m_source->GetTextRange(startpos, endpos);
    if(word == "")
    {
      startpos = m_source->WordStartPosition(pos + 1, true);
      endpos = m_source->WordEndPosition(pos + 1, true);
      word = m_source->GetTextRange(startpos, endpos);
    }
    if(startpos > 0)
      chr = m_source->GetTextRange(startpos - 1, startpos);
  }
  if(word != "")
  {
    if(chr == wxT("$"))
      word = wxT("$") + word;
    return true;
  }
  return false;
}

// Events
void panSourceView::OnMarginClick(wxStyledTextEvent& evt)
{
  if(evt.GetMargin() == MarginActiveLine)
    ToggleBookmark(m_source->LineFromPosition(evt.GetPosition()));
  else if(evt.GetMargin() == MarginBreakpoint)
    ToggleBreakpoint(m_source->LineFromPosition(evt.GetPosition()));
}

void panSourceView::OnLeftClick(wxStyledTextEvent& evt)
{

  wxString word = m_source->GetTextRange(m_source->WordStartPosition(evt.GetPosition(), true), m_source->WordEndPosition(evt.GetPosition(), true) );
  if(word == "")
    word = m_source->GetTextRange(m_source->WordStartPosition(evt.GetPosition() + 1, true), m_source->WordEndPosition(evt.GetPosition() + 1, true) );

  if(word != "")
    wxGetApp().SendCommand(wxT("getvariable"), CompileHash("variable", wxString("$" + word).c_str(), 0));

  return;
}


void panSourceView::OnRightClick(wxMouseEvent& event)
{
  wxString strWord;

  if(CheckForWord(event.GetPosition(), strWord))
  {
    wxMenu menu(strWord);
    menu.Append(cmd_variable_inspect, "Sho&w");
    menu.Append(cmd_variable_watch, "&Watch");
    menu.Append(cmd_variable_set, "&Set value");
    menu.Append(cmd_variable_break, "&Break when...");

    strPopupWord = strWord;

    PopupMenu(&menu, event.GetPosition().x, event.GetPosition().y);


    return;
  }

  event.Skip();
  return;
}


void panSourceView::OnWatch(wxCommandEvent& event)
{
  wxGetApp().AddWatch(strPopupWord);
}

void panSourceView::OnView(wxCommandEvent& event)
{
  wxGetApp().SendCommand(wxT("getvariable"), CompileHash("variable", strPopupWord.c_str(), 0));
}

void panSourceView::OnSet(wxCommandEvent& event)
{

  wxTextEntryDialog *teValue =
    new wxTextEntryDialog(
      wxGetApp().GetGubedWindow(),
      _("Enter value for: ") + strPopupWord,
      _("Set variable contents"),
      wxT(""));

  if(teValue->ShowModal() ==  wxID_OK)
    wxGetApp().SendCommand(wxT("setvariable"), CompileHash("variable", strPopupWord.c_str(), "value", teValue->GetValue().c_str(),  0));

}


void panSourceView::OnCondition(wxCommandEvent& event)
{
  frmConditionalBreakpoint *condition;

  condition = new frmConditionalBreakpoint(wxGetApp().GetGubedWindow());
  condition->SetFilename(strSourcename);
  condition->SetExpression(strPopupWord);

  if(condition->ShowModal() ==  wxID_OK)
  {
    if(condition->GetExpression() != _T("")) 
    {
      Breakpoint bp(condition->GetType(), 
                    condition->GetExpression(),
                    condition->GetFilename(),
                    condition->GetClass(),
                    condition->GetFunction()
                    );
      wxGetApp().setBreakpoint(bp);
    }
  }
  delete condition;

/*
  wxTextEntryDialog * condition = new wxTextEntryDialog(
                                    this,
                                    _("Specify what expression you wish to break at\n(it will break when the result of expression changes)"),
                                    _("Conditional breakpoint"),
                                    strPopupWord);

  if(condition->ShowModal() ==  wxID_OK)
  {
    if(condition->GetValue() != _T(""))
      wxGetApp().ConditionalBreakpoint(condition->GetValue());
  }*/
}


#endif //HAVE_STC
