/*
 * Copyright (C) 2004 by Linus McCabe
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#ifdef __WIN32__
	#include "../configmingw.h"
#else
	#include "../config.h"
#endif
#ifndef panSourceView_H
#define panSourceView_H

#ifdef HAVE_STC

#include "wxClientGlobal.h"
#include <wx/stc/stc.h>

class gbdStyledTextCtrl: public wxStyledTextCtrl
{
  public:
    gbdStyledTextCtrl (wxWindow *parent, wxWindowID id = -1,
          const wxPoint &pos = wxDefaultPosition,
          const wxSize &size = wxDefaultSize,
          long style = wxSUNKEN_BORDER|wxVSCROLL
         );
    ~gbdStyledTextCtrl();

    virtual void OnMouseRightUp(wxMouseEvent& evt);

    private:
      DECLARE_EVENT_TABLE()
};

class panSourceView: public wxPanel
{
  private:
    gbdStyledTextCtrl *m_source;
    long lngCurrentSourceLine;

    // General functions
    void SetupControls(void);

    bool CheckForWord(const wxPoint&, wxString&);

    // Events
    void OnLeftClick(wxStyledTextEvent& event);

    void OnRightClick(wxMouseEvent& event);
    /*void OnRangeSelect(wxGridRangeSelectEvent&);
    void OnCellSelect(wxGridEvent&);*/
    void OnMarginClick(wxStyledTextEvent&);
    void OnView(wxCommandEvent&);
    void OnWatch(wxCommandEvent&);
    void OnSet(wxCommandEvent&);
    void OnCondition(wxCommandEvent&);

    wxTextCtrl *txtSourceName;
    wxTextCtrl *txtCurrentline;
    wxTextCtrl *txtTotallines;

    wxString strSourcename;
    wxString strIdentifier;
    wxString strPopupWord;

    //int intPage;
    int intFontWidth;

  public:
    bool SetIdentifier(const wxString &);
    void SetSource(wxString Source);
    void ShowActiveSourceLine();
    void ShowSourceLine(long);
    void ToggleBookmark(long);
    void ToggleBreakpoint(long);
    void SetBreakpoint(long, bool);
    void ClearBreakpoint(long, bool);
    void SetActiveSourceLine(long);
    //void SetRowColour(long);

    const wxString &GetFilename() { return strSourcename;};

  //  void SetPage(int intPage);
    int GetPage();

    panSourceView(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, const wxString&);
    DECLARE_EVENT_TABLE()

    enum {
      MarkerBookmark = 0,
      MarkerBreakpoint,
      MarkerActiveLine
    };
    enum {
      MarginLinenum = 0,
      MarginActiveLine,
      MarginBreakpoint
    };
};

enum {
  cmd_variable_inspect = 5999,
  cmd_variable_watch ,
  cmd_variable_set,
  cmd_variable_break
};

#endif //HAVE_STC
#endif //panSourceView_H
