/*
 * sockLineSocket.cpp
 * Classes to process one line at a time from sockets
 *
 * Original code by Brian Victor, heavily rewritten by now :)
 * Copyright 2003 Brian Victor
 * Released under the terms of the wxWindows license
 *
*/

#include "sockLineSocket.h"
#include "appGubed.h"
#include "nbSources.h"
#include "Breakpoint.h"
#include <wx/config.h>

BEGIN_EVENT_TABLE(wxSocketLineHandler, wxEvtHandler)
  EVT_SOCKET(-1, wxSocketLineHandler::OnSocketRead)
END_EVENT_TABLE()


wxSocketLineHandler::wxSocketLineHandler()
{
  wxConfigBase *cfgGubed = wxConfigBase::Get();

  m_socket = NULL;
  m_server = NULL;
  m_client = NULL;
  m_datalen = -1;
  
  wxIPV4address addr = getAddress();
  if(!cfgGubed->Read(wxT("UseProxy"), 0l))
  {
    ::wxLogDebug(_("Creating listening socket"));
    m_server = new wxSocketServer(addr);
    m_server->SetEventHandler(*this, wxSocketLineHandler::Server);
    m_server->SetNotify(wxSOCKET_INPUT_FLAG | wxSOCKET_CONNECTION_FLAG | wxSOCKET_LOST_FLAG | wxSOCKET_OUTPUT_FLAG);
    m_server->SetFlags(wxSOCKET_NOWAIT);
    m_server->Notify(true);
    if(!m_server->Ok())
      ::wxLogWarning(_("Unable to bind to port %ld"), (long)addr.Service());
  }
  else
  {
    ::wxLogDebug(_("Creating connecting socket"));
    m_client = new wxSocketClient(wxSOCKET_NOWAIT);
    m_client->SetEventHandler(*this, wxSocketLineHandler::Client);
    m_client->SetNotify(wxSOCKET_INPUT_FLAG | wxSOCKET_CONNECTION_FLAG | wxSOCKET_LOST_FLAG | wxSOCKET_OUTPUT_FLAG);
    m_client->Notify(true);
    m_client->Connect(addr, false);
  }
  wxGetApp().setConnectionState();
}

wxSocketLineHandler::~wxSocketLineHandler()
{
  if(m_socket && m_socket != m_client )
  {
    SendCommand(wxT("die"), 0);
    m_socket->Close();
    delete m_socket;
    m_socket = NULL;
  }
  if(m_client)
  {
    SendCommand(wxT("die"), 0);
    m_client->Close();
    delete m_client;
    m_client = NULL;
    m_socket = NULL;
  }
  if(m_server)
  {
    delete m_server;
    m_server = NULL;
  }
}

wxIPV4address wxSocketLineHandler::getAddress()
{
  wxConfigBase *cfgGubed = wxConfigBase::Get();
  wxString strPort = cfgGubed->Read(wxT("Port"), wxT("8016"));
  wxString host = cfgGubed->Read(wxT("Server"), wxT(""));

  wxIPV4address address;
  address.Service(strPort);
  if(cfgGubed->Read(wxT("UseProxy"), 0l))
  {
    if(host == "")
      host = "localhost";
    address.Hostname(host);
  }

  return address;
}

void wxSocketLineHandler::Connect()
{
  if(m_client)
  {
    if(isConnected())
    {
      SendCommand(wxT("die"), 0);
      m_socket->Close();// We cant delete m_socket here, cause it's really m_client!!
      m_socket = NULL;
    }
    else
    {
      ::wxLogDebug(_("Creating connecting socket"));
      wxIPV4address addr = getAddress();
      m_client->Connect(addr, false);
    }
  }
  else if(m_server)
  {
    if(isConnected())
    {
      SendCommand(wxT("die"), 0);
      m_socket->Close();
      delete m_socket;
      m_socket = NULL;
    }
    else
    {
      if(m_server)
        delete m_server;

      wxIPV4address addr = getAddress();
      m_server = new wxSocketServer(addr);
      m_server->SetEventHandler(*this, wxSocketLineHandler::Server);
      m_server->SetNotify(wxSOCKET_INPUT_FLAG | wxSOCKET_CONNECTION_FLAG | wxSOCKET_LOST_FLAG | wxSOCKET_OUTPUT_FLAG);
      m_server->SetFlags(wxSOCKET_NOWAIT);
      m_server->Notify(true);
      if(!m_server->Ok())
        ::wxLogWarning(_("Unable to bind to port %ld"), (long)addr.Service());

    }
  }
  wxGetApp().setConnectionState();
}

void wxSocketLineHandler::OnSocketRead(wxSocketEvent& ev)
{

  switch (ev.GetSocketEvent())
  {

    case wxSOCKET_INPUT:
      {
        ::wxLogDebug(_("wxSocketLineHandler::OnSocketRead, wxSOCKET_INPUT: %d"), ev.GetId());
        wxString strData;
        char buf[1024];

        // Data comes in this format:
        // command:datalength;data
        while (1)
        {
          m_socket->Read(buf, 1024);
          m_buf.Append(buf, m_socket->LastCount());

          // If datalen == -1, we didnt read the command yet, otherwise were reading data.
          if(m_datalen == -1)
          {
            strData = m_buf.BeforeFirst(';');
            m_buf = m_buf.AfterFirst(';');
            if (strData == m_buf)
              break;
            strCommand = strData.BeforeFirst(':');
            strData.AfterFirst(':').ToLong(&m_datalen);
          }
          if(m_datalen != -1 && (int)m_buf.Len() >= m_datalen)
          {
            strData = m_buf.Left(m_datalen);
            m_buf = m_buf.Mid(m_datalen);

            HandleLine(ev.GetId(), strCommand, strData);

            m_datalen = -1;
          }
        }
      }
      break;

    case wxSOCKET_CONNECTION:
      {
        ::wxLogDebug(_("wxSocketLineHandler::OnSocketRead, wxSOCKET_CONNECTION: %d"), ev.GetId());
        if(ev.GetId() == wxSocketLineHandler::Server)
        {
          if(!m_socket)
          {
            ::wxLogInfo(_("Client accepted"));
            m_socket = m_server->Accept(false);
            if (m_socket)
            {
              m_socket->SetEventHandler(*this, wxSocketLineHandler::Socket);
              m_socket->SetNotify(wxSOCKET_INPUT_FLAG | wxSOCKET_CONNECTION_FLAG | wxSOCKET_LOST_FLAG | wxSOCKET_OUTPUT_FLAG);
              m_socket->SetFlags(wxSOCKET_NOWAIT);
              m_socket->Notify(true);
              SendCommand(wxT("wait"), 0);
              wxGetApp().DebugMode(true);
            }
          }
        }
        else if(ev.GetId() == wxSocketLineHandler::Client)
        {
          if(!m_socket)
          {
            ::wxLogInfo(_("Client connected"));
            m_socket = m_client;
            SendCommand(wxT("wait"), 0);
            wxGetApp().DebugMode(true);
          }
        }
      }
      wxGetApp().setConnectionState();
      break;

    case wxSOCKET_LOST:
      {
        ::wxLogDebug(_("wxSocketLineHandler::OnSocketRead, wxSOCKET_LOST: %d"), ev.GetId());
        ::wxLogInfo(_("Client disconnected"));
        wxGetApp().DebugMode(false);

        // Do we have a waiting connection?
        if(m_server)
        {
          // Delete old socket
          delete m_socket;

          m_socket = m_server->Accept(false);
          if (m_socket)
          {
            m_socket->SetEventHandler(*this, wxSocketLineHandler::Socket);
            m_socket->SetNotify(wxSOCKET_INPUT_FLAG | wxSOCKET_CONNECTION_FLAG | wxSOCKET_LOST_FLAG | wxSOCKET_OUTPUT_FLAG);
            m_socket->SetFlags(wxSOCKET_NOWAIT);
            m_socket->Notify(true);
            SendCommand(wxT("wait"), 0);
          }
        }
        else
        {
          // No delete m_socket here!! (its really m_client)
          m_socket = NULL;
        }

      }

      wxGetApp().setConnectionState();
      break;

    case wxSOCKET_OUTPUT:
      {
        ::wxLogDebug(_("wxSocketLineHandler::OnSocketRead, wxSOCKET_OUTPUT: %d"), ev.GetId());
      }
      break;

    default:
      ::wxLogDebug(_("wxSocketLineHandler::OnSocketRead, default"));

      break;
  }
}

bool wxSocketLineHandler::isListening()
{
  return m_server && m_server->Ok();
}

void wxSocketLineHandler::SendCommand(const wxString& command, StringHash args)
{
  if(m_socket)
  {
    wxString buffer = phpSerialize(args);
    ::wxLogDebug(_("Sending: ") + command + ", " + buffer.Left(280));
    
    buffer = command + wxT(":") + wxString::Format(wxT("%d"), buffer.Length()) + ";" + buffer;    
    m_socket->Write(buffer.c_str(), buffer.length());
  }
  else
    ::wxLogDebug(_("Not connected!"));
}

void wxSocketLineHandler::SendCommand(const wxString& command, char * firstarg, ...)
{
  StringHash ca;
  char *next;
  
  va_list l_Arg;
  va_start(l_Arg, firstarg);
  
  next = firstarg;
  while(next)
  { 
    ca[(wxString)next] = (wxString)va_arg(l_Arg, char*) ; 
    wxLogDebug(_T("Added arg/valuepair '%s' : '%s'"), next, ca[next].Left(30).c_str());
    
    next = va_arg(l_Arg, char*);
  }
  
  va_end(l_Arg);
  
  SendCommand(command, ca);

}

void wxSocketLineHandler::HandleLine(int sockid, const wxString &strComm, const wxString &rawdata)
{
  StringHash args = ParseArgs(rawdata);
  
  ::wxLogDebug(wxString::Format(_("Received command \"%s\" with data \"%s\""), strComm.c_str(), (rawdata.Length() > 280 ? rawdata.Left(277) + "..." : rawdata.Left(280)).c_str()));
  
  /****s* Script.to.Client/commandme
   * NAME
   *    commandme
   *
   * ARGS
   *    * filename - current file
   *    * line - current line of execution
   *      
   * DESCRIPTION
   *    tells the client the script is halted (For any reason.
   *    Can also be during 'trace') and needs to be updated.
   *    The client should update it's views and request watches,
   *    and backtrace.
   *    Then either reply with a "wait" or "next" command.
   *
   * SEE ALSO
   *    wait, next, sendwatches, sendbacktrace
   *
   ****/
  if(strComm == "commandme") {

    wxGetApp().GetGubedWindow()->GetSources()->SetActiveSourceLine(args["filename"], ::wxAtoi(args["line"]));
    wxGetApp().SendWatches();
    SendCommand(wxT("sendbacktrace"), 0);
    if(wxGetApp().executionState() == ExecutionStates::Trace)
      SendCommand(wxT("wait"), 0);

    if(wxGetApp().executionState() != ExecutionStates::Pause)
      SendCommand(wxT("next"), 0);

    return;
  }
  
  /****s* Script.to.Client/sendbreakpoints
   * NAME
   *    sendbreakpoints
   *
   * ARGS
   *    none
   *      
   * DESCRIPTION
   *    Tells the client the script needs to know about all
   *    breakpoints the client knows of. Usually takes place
   *    when a new debugging session starts.
   *
   *    The client should respond by sending all breakpoints
   *    and conditional breakpoints one by one.
   *
   * SEE ALSO
   *    breakpoint, conditionalbreakpoint
   *
   ****/
  else if(strComm == "sendbreakpoints")
  {
    wxGetApp().sendBreakpoints();
  }
  
  /* OBSOLETE ***s* Script.to.Client/unparsed
   * NAME
   *    unparsed
   *
   * ARGS
   *    * filename - what file it is thats about to be parsed
   *      
   * DESCRIPTION
   *
   *    Tells the client a script is about to be parsed. 
   *
   *    This command is probably obsolete since the parsefailed
   *    command was introduced.
   *
   * SEE ALSO
   *    parsefailed, parseok
   *
   ****/
//   else if(strComm == "unparsed")
//   {
//     wxGetApp().GetGubedWindow()->GetSources()->UnParsed(args["filename"]);
//     return;
//   }
  
  /* OBSOLETE ***s* Script.to.Client/parseok
   * NAME
   *    parseok
   *
   * ARGS
   *    * filename - what file it is that was parsed ok
   *      
   * DESCRIPTION
   *
   *    This command tells the client a script was parsed
   *    ok.
   *
   *    This command is probably obsolete since the parsefailed
   *    command was introduced.
   *
   * SEE ALSO
   *    unparsed, parsefailed
   *
   ****/
//   else if(strComm == "parseok")
//   {
//     wxGetApp().GetGubedWindow()->GetSources()->ParsedOk(args["filename"]);
//     return;
//   }
  /****s* Script.to.Client/forcebreak
   * NAME
   *    forcebreak
   *
   * ARGS
   *    none
   *      
   * DESCRIPTION
   *
   *  This command tells the client execution is halted due to a 
   *  forced (ie, hardcoded \* STOP *\) breakpoint.
   *
   *  The client should set its internal execution status to paused.
   *
   * SEE ALSO
   *
   ****/
  else if(strComm == "forcebreak")
  {
    wxGetApp().setExecutionState(ExecutionStates::Pause);
    wxGetApp().GetGubedWindow()->Show(true);
    wxGetApp().GetGubedWindow()->Raise();
    ::wxLogMessage(_("Breakpoint reached"));
  }
  /****s* Script.to.Client/conditionalbreak
   * NAME
   *    conditionalbreak
   *
   * ARGS
   *    none
   *      
   * DESCRIPTION
   *
   *  Execution is halted due to a conditional breakpoint and
   *  the client should set its internal execution status to 
   *  paused.
   *  
   *  The status of all conditional breakpoints will follow.
   *
   * SEE ALSO
   *
   ****/
  else if(strComm == "conditionalbreak")
  {
    wxGetApp().setExecutionState(ExecutionStates::Pause);
    wxGetApp().GetGubedWindow()->Show(true);
    wxGetApp().GetGubedWindow()->Raise();
    ::wxLogMessage(_("Result of conditional breakpoint expression changed"));
  }
  /****s* Script.to.Client/source
   * NAME
   *    source
   *
   * ARGS
   *    * filename - the filename of the sent source
   *    * sourcecode - the sourcecode of the file
   *      
   * DESCRIPTION
   *    This commands sends the sourcecode of a file to the client.
   *  
   * SEE ALSO
   *    sendsource
   ****/
  else if(strComm == "source")
  {
    wxGetApp().GetGubedWindow()->GetSources()->SetSource(args["filename"], args["sourcecode"]);
    return;

  }
  /****s* Script.to.Client/setactiveline
   * NAME
   *    setactiveline
   *
   * ARGS
   *    * filename - current file
   *    * line - current line of exection
   *    * mode - tells the client what mode its urrently in: pause, run or trace (optional)
   *
   * DESCRIPTION
   *    Tells the client the current file and line of execution
   *
   * SEE ALSO
   *
   ****/
  else if(strComm == "setactiveline")
  {
    wxGetApp().GetGubedWindow()->GetSources()->SetActiveSourceLine(args["filename"], ::wxAtoi(args["line"]));
    if(args["mode"] == "pause")
      wxGetApp().setExecutionState(ExecutionStates::Pause);

    return;
  }
  /****s* Script.to.Client/getrunmode
   * NAME
   *    getrunmode
   *
   * ARGS
   *    none
   *      
   * DESCRIPTION
   *    Upon receiving this command, the client should send the
   *    current run mode (pause, run, trace) as well
   *    as current error mask (seterrormask)
   *
   *    This is part of the initial communication when a script started
   *    to execute and it needs to know what run mode the user has requested
   *    as well as what errors to halt on.
   *
   * SEE ALSO
   *    pause, run, trace, seterrormask
   ****/
  else if(strComm == "getrunmode")
  {
    wxGetApp().setDisplayDelay(-1);
    if(wxGetApp().executionState() == ExecutionStates::Pause)
      SendCommand(wxT("pause"), 0);
    else if(wxGetApp().executionState() == ExecutionStates::Run)
      SendCommand(wxT("run"), 0);
    else if(wxGetApp().executionState() == ExecutionStates::Trace)
      SendCommand(wxT("trace"), 0);

    wxConfigBase *cfgGubed = wxConfigBase::Get();
    SendCommand(wxT("seterrormask"), "errormask", cfgGubed->Read("ErrorMask", "0").c_str(), 0 );
    return;
  }
  /****s* Script.to.Client/parsetime
   * NAME
   *    parsetime
   *
   * ARGS
   *    * filename - what file was parsed
   *    * time - parse time in ms 
   *      
   * DESCRIPTION
   *    This command is informational only and tells the client
   *    how long time it took to parse a file.
   *    
   *    Should be displayed in verbose mode only.
   *
   * SEE ALSO
   *
   ****/
  else if(strComm == "parsetime")
  {
    ::wxLogVerbose(wxString::Format(_T("Parsed \"%s\" in %s ms"), args["filename"].c_str(), args["time"].c_str()));
    return;
  }
  /****s* Script.to.Client/status
   * NAME
   *    status
   *
   * ARGS
   *    * message - clear text message to be displayed
   *    * args - number of args in the message string (%X%)
   *    * arg1 - first arg to replace %1%
   *    * argX - etc
   *      
   * DESCRIPTION
   *    A message from Gubed to the user. In interntionalizationable  
   *    english. String can hold placeholders like %1% ... %X% that will be
   *    replaces by arg1 ... argX
   *
   * SEE ALSO
   *
   ****/
  else if(strComm == "status")
  {
    int argcnt = wxAtoi(args["args"]);
    wxString msg = _(args["message"]);
    for(int cnt = 1; cnt <= argcnt; cnt++)
      msg.Replace(wxString::Format(_T("%%%d%%"), cnt), args[wxString::Format(_T("arg%d"), cnt)], true);

    ::wxLogMessage(msg);
    return;
  }
  /****s* Script.to.Client/processouput
   * NAME
   *    processoutput
   *
   * ARGS
   *    * output - what output a phpprocess command generated
   *      
   * DESCRIPTION
   *    Returns the output of a phpprocess command previously 
   *    sent.
   *
   * SEE ALSO
   *
   ****/
  else if(strComm == "processoutput")
  {
    ::wxLogMessage((wxString)_("Output:") + _T("\n") + args["output"]);
    wxWindow* txt = wxWindow::FindWindowById(cmd_commandtext, wxGetApp().GetGubedWindow());
    if(txt)
      txt->SetFocus();
    return;
  }
  /****s* Script.to.Client/parsefailed
   * NAME
   *    parsefailed
   *
   * ARGS
   *    * filename - name of the file that failed to parse.
   *      
   * DESCRIPTION
   *    Tells the client and execution was halted (ie killed) because of
   *    a parse error. 
   *
   *    This either means there are errors in the php code, or that Gubed's
   *    parser was unable to parse the file(s).
   *
   * SEE ALSO
   *    
   *
   ****/
  else if(strComm == "parsefailed")
  {
    ::wxLogWarning(wxString::Format(_T("Parsed failed for file \"%s\", please verify that it does not contain errors!"), args["filename"].c_str()));
    return;
  }
  /****s* Script.to.Client/end
   * NAME
   *    end
   *
   * ARGS
   *    * filename - name of the file that reached the end
   *      
   * DESCRIPTION
   *    Tells the client the end of an include file was reached.
   *  
   *    The client should forward this information to the user in
   *    verbose mode.
   *
   * SEE ALSO
   *
   ****/
  else if(strComm == "end")
  {
    ::wxLogVerbose(_("At end of include '%s'"), args["filename"].c_str());
    return;

  }
  /****s* Script.to.Client/watch
   * NAME
   *    watch
   *
   * ARGS
   *    * variable - contents of a php variable that was on the watch list, in serialized form
   *      
   * DESCRIPTION
   *    This command tells the client the contents of a variable on the watch list so
   *    that the ui can be updated to reflect the changes.
   *
   *    The variable comes in PHP's serialized form, see http://php.net/serialize
   *
   * SEE ALSO
   *    variable, http://php.net/serialize
   *
   ****/
  else if(strComm == "watch")
  {
    wxGetApp().GetGubedWindow()->ShowWatch(args["variable"]);
    return;

  }
  /****s* Script.to.Client/variable
   * NAME
   *    variable
   *
   * ARGS
   *    * variable - contents of a php variable that was requested
   *
   * DESCRIPTION
   *    This command tells the client the value of a variable.
   *
   *    The data comes in PHP's serialized form, see http://php.net/serialize
   *
   * SEE ALSO
   *    http://php.net/serialize
   ****/
  else if(strComm == "variable")
  {
    wxGetApp().GetGubedWindow()->ShowVariable(args["variable"]);
    return;

  }
  /****s* Script.to.Client/backtrace
   * NAME
   *    backtrace
   *
   * ARGS
   *    * backtrace - backtrace in the form of a serialized array
   *      
   * DESCRIPTION
   *    Informs the client of the current backtrace
   *
   * SEE ALSO
   *    http://php.net/serialize
   *
   ****/
  else if(strComm == "backtrace")
  {
    wxGetApp().GetGubedWindow()->ShowBacktrace(args["backtrace"]);
    return;

  }
  /****s* Script.to.Client/showcondition
   * NAME
   *    showcondition
   *
   * ARGS
   *    * type - either "change" or "true", if it is a break on changed expression or when expression is true
   *    * filename - an optional filename and halting will only occur if execution is currently in that file
   *    * class - only break if the condition is true and in a method of an object of this class 
   *    * function - only break if the condition is true and in a function name like this 
   *    * expression - what expression to evaluate, for example ($a == 12)
   *    * value - what value the expression currently evaluates to
   *
   * DESCRIPTION
   *    Tells the client about the condition (value) of a conditional breakpoint expression.
   *
   * SEE ALSO
   *
   ****/
  else if(strComm == _T("showcondition"))
  {
    Breakpoint bp(Breakpoint::True, args[_T("expression")], args[_T("filename")], args[_T("class")], args[_T("function")]);
    bp.setValue(args[_T("value")]);
    bp.setLine(wxAtoi(args[_T("line")]));
    
    // Set correct type
    if(args[_T("type")] == _T("line"))
        bp.setType(Breakpoint::Line);
    else if(args[_T("type")] == _T("change"))
        bp.setType(Breakpoint::Change);
    
    // Update ui
    wxGetApp().updateBreakpoint(bp);
    return;

  }
  /****s* Script.to.Client/sentwatches
   * NAME
   *    sentwatches
   *
   * ARGS
   *    key - id key
   *      
   * DESCRIPTION
   *    Tells the client all watches are sent, in case it
   *    needs to do cleaning up or similar.
   *
   * SEE ALSO
   *    watch
   *
   ****/
  else if(strComm == "sentwatches")
  {
    wxGetApp().GetGubedWindow()->PrepareWatches(::wxAtoi(args["key"]), true);
    return;

  }
  /****s* Script.to.Client/protocolversion
   * NAME
   *    protocolversion
   *
   * ARGS
   *    * version - what protocol version the script uses
   *      
   * DESCRIPTION
   *    To make sure the client and the script speaks the same language,
   *    the scripts tells the client what version of the protocol it's using.
   *    The client should compare this to it's own internal version and
   *    kill the script if they do not match.
   *
   * SEE ALSO
   *
   ****/
  else if(strComm == "protocolversion")
  {
    if(args["version"] != wxT(PROTOCOLVER))
    {
      wxLogWarning( _("The script being debugged does not communicate with the correct protocol version"));
      SendCommand(wxT("die"), 0);
    }
    return;

  }
  /****s* Script.to.Client/initialize
   * NAME
   *    initialize
   *
   * ARGS
   *    * filename - filename of the script entered
   *    * checksum - checksum of the script entered (using this, the
   *      the client can tell if it needs to refresh it's 
   *      source view)
   *
   * DESCRIPTION
   *    initialize tells the client a new script was entered and
   *    needs to be displayed. The client can then chose to load
   *    the script from disk or request it's sourcecode from Gubed.
   *
   *    The client should reply with either a 'havesource' or a 
   *    'sendsource' command, to indicate if it needs the source sent
   *    or not.
   *
   ****/
  else if(strComm == "initialize")
  {
    ::wxLogVerbose(_("Established connection to %s"), args["filename"].c_str());
    SendCommand(wxT("sendprotocolversion"), 0);

    if(wxGetApp().GetGubedWindow()->GetSources()->AddSource(args["filename"], args["checksum"]))
      SendCommand(wxT("havesource"), "filename", args["filename"].c_str(), 0);
    else
      SendCommand(wxT("sendsource"), "filename", args["filename"].c_str(), 0);
    return;

  }
  /****s* Script.to.Client/error
   * NAME
   *    error
   *
   * ARGS
   *    * filename - file where the error occurred
   *    * line - code line where the error occurred
   *    * errnum - php error number of the error
   *    * errmsg - php error message of the error
   *      
   * DESCRIPTION
   *    Tells the client an error occurred in the script and
   *    that execution is halted because of it. Which kind of
   *    errors halts execution depends on the current settings.
   *
   *    the arguments are currently separated by colons
   *
   ****/
  else if(strComm == "error")
  {
    // /mnt/hdc1/projects/Gubed/Gubed/GubedTest/classes.php:12:8:Undefined index;  test
    long errortype, errormask;

    errortype = ::wxAtoi(args["errnum"]);

    ::wxLogWarning(wxT("%s, line %s, error %s: %s"), args["filename"].c_str(), args["line"].c_str(), args["errnum"].c_str(), args["errmsg"].c_str());

    wxConfigBase *cfgGubed = wxConfigBase::Get();
    cfgGubed->Read("ErrorMask", "0").ToLong(&errormask);

    if(errormask & errortype)
    {
      wxGetApp().setExecutionState(ExecutionStates::Pause);
      wxGetApp().GetGubedWindow()->Show(true);
      wxGetApp().GetGubedWindow()->Raise();
    }
    else if(wxGetApp().executionState() == ExecutionStates::Trace)
      wxGetApp().setExecutionState(ExecutionStates::Trace);
    else if(wxGetApp().executionState() == ExecutionStates::Run)
      wxGetApp().setExecutionState(ExecutionStates::Run);
    else
      wxGetApp().setExecutionState(ExecutionStates::Pause);

    return;

  }
  else
    ::wxLogWarning(_("Unknown command '%s' with data '%s'"), strComm.c_str(), rawdata.c_str());
}

StringHash wxSocketLineHandler::ParseArgs(const wxString &args)
{
  StringHash ca;
  long cnt, length;
  
  ::wxLogDebug(_("Parsing '%s'"), args.c_str());
  // a:2:{s:4:"name";s:7:"Jessica";s:3:"age";s:2:"26";s:4:"test";i:1;}

  // No args
  if(args == _T("") || args == _T("a:0:{}"))
    return ca;
  
  // Make sure we have a good string
  if(args.Left(2) != wxT("a:"))
  {
    ::wxLogError(_("An error occurred in the communication link, data received was '%s'"), args.c_str());
    return ca;
  }
  
  args.Mid(2, args.Find('{') - 3).ToLong(&cnt);  
  wxString data = args.AfterFirst('{');
  
  wxString tmp, func;
  while(cnt > 0) 
  {
    tmp = data.BeforeFirst('"');
    tmp.Mid(2, tmp.Len() - 3).ToLong(&length);
    
    func = data.Mid(tmp.Len() + 1, length);
    data = data.Mid( tmp.Len() + length + 3);
    
    if(data.Left(1) == 'i')
    {
      // Integer data
      tmp = data.AfterFirst(':').BeforeFirst(';');
      ca[func] = tmp;
      data = data.Mid(tmp.Len() + 3);
    }
    else
    {
      // String data
      tmp = data.BeforeFirst('"');
      tmp.Mid(2, tmp.Len() - 3).ToLong(&length);
      
      ca[func] = data.Mid(tmp.Len() + 1, length);
      data = data.Mid( tmp.Len() + length + 3);
    }
    
    cnt--;
  }

  return ca;
}

wxString wxSocketLineHandler::phpSerialize(StringHash args)
{
  StringHash::iterator it;
  // a:2:{s:4:"name";s:7:"Jessica";s:3:"age";s:2:"26";s:4:"test";i:1;}
  wxString ret = wxString::Format(wxT("a:%d:{"), args.size()) ;
  for( it = args.begin(); it != args.end(); ++it )
  {
    if(it->second.IsNumber() && !it->second.IsEmpty())
      ret += wxString::Format(wxT("s:%d:\"%s\";i:%s;"), it->first.length(), it->first.c_str(), it->second.c_str());
    else
      ret += wxString::Format(wxT("s:%d:\"%s\";s:%d:\"%s\";"), it->first.length(), it->first.c_str(), it->second.length(), it->second.c_str());
  }

  ret += wxT("}");
  return ret;
}
