/*
 * Copyright (C) 2002 - 2004 by Linus McCabe
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#include "gbdserver.h"
#include "gbdNetwork.h"

gbdNetwork::gbdNetwork(int intPort)
{
  int intYes = 1;
  char chYes = 'Y';
  struct sockaddr_in addrListen;

  intComm = 0;
  intListen = socket(AF_INET, SOCK_STREAM, 0);
  if(intListen < 0)
  {
    LOG_ERROR "Unable to setup socket on port " << intPort << "\n";
    return;
  }

  // not really needed?
  // fcntl(intListen, F_SETFL, O_NONBLOCK);
#ifdef __WIN32__
  if(setsockopt(intListen, SOL_SOCKET, SO_REUSEADDR, &chYes, sizeof(int)) == -1)
  {
#else
  if(setsockopt(intListen, SOL_SOCKET, SO_REUSEADDR, &intYes, sizeof(int)) == -1)
  {
#endif
    LOG_ERROR "Unable to alter socket, port: " << intPort << ", socket: " << intListen << "\n";
    return;
  }

  addrListen.sin_family = AF_INET;
  addrListen.sin_addr.s_addr = INADDR_ANY;
  addrListen.sin_port = htons(intPort);
  if(bind(intListen, (struct sockaddr *)&addrListen, sizeof(addrListen)) == -1)
  {
    LOG_ERROR "Unable to bind to port " << intPort << "\n";
    return;
  }

  if(listen(intListen, 0) == -1)
  {
    LOG_ERROR "Unable to listen on socket\n";
    return;
  }

  if(intListen > 0)
    LOG_INFO "Awaiting connection on port " << intPort << "\n";

  intThisPort = intPort;
}


gbdNetwork::~gbdNetwork()
{

  if(intListen > 0)
  {
    LOG_INFO "Closing connection on port " << intThisPort << "\n";
#ifdef __WIN32__
    closesocket(intListen);
#else
    close(intListen);
#endif

  }

}

void gbdNetwork::AddSockets(fd_set* fdpSet)
{

  fdpMaster = fdpSet;

  if(intListen > 0 && intThisPort > 0)
    FD_SET(intListen, fdpSet);

  return;
}

void gbdNetwork::Accept ()
{
  int intSize;
  int intTemp;

  intSize = sizeof(struct sockaddr_in);
#ifdef __WIN32__
  intComm = accept(intListen,  (sockaddr*)&addrRemote, &intSize);
#else
  intComm = accept(intListen,  (sockaddr*)&addrRemote, (socklen_t *)&intSize);
#endif

  if(intComm == -1)
  {
    LOG_ERROR "Error accepting connection on " << intThisPort << "\n";
    return;
  }

  FD_SET(intComm, fdpMaster);
  ProcessAccept(intThisPort, true);

}

void gbdNetwork::HandleInput(fd_set* fdpSet)
{

  if(FD_ISSET(intListen, fdpSet))
    Accept();

  if(intComm > 0 && FD_ISSET(intComm, fdpSet))
  {
    char buffer[1024];
    int intBytes;
    string strRead;
    size_t stLength;

    intBytes = recv(intComm, buffer, sizeof(buffer) - 1, 0);
    buffer[intBytes] = 0;

    if(intBytes < 0)
    {
      LOG_ERROR "Recv error on " << intThisPort << "\n";

    }
    else if(intBytes == 0)
    {
      LOG_INFO "Connection closed on " << intThisPort << "\n";
      FD_CLR(intComm, fdpMaster);
#ifdef __WIN32__
      closesocket(intComm);
#else
      close(intComm);
#endif
      intComm = 0;
      ProcessAccept(intThisPort, false);

    }
    else
    {
      strRead = buffer;
      m_buffer.append(strRead);
      ::HaveData(intThisPort);
    }
  }

}


int gbdNetwork::GetMaxSocket()
{
  return max(intComm, intListen);
}

void gbdNetwork::SendData(string data)
{
  LOG_DEBUG "Sending data to port " << intThisPort << ":\n" << data << "\n\n";
  send(intComm, data.c_str(), data.size(), 0);
}

string gbdNetwork::FlushBuffer()
{
  string data = m_buffer;
  m_buffer = "";
  return data;
}

bool gbdNetwork::IsConnected()
{
  return intComm > 0;
}
