/*
 * Copyright (C) 2002 - 2004 by Linus McCabe
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "gbdserver.h"
#include "gbdNetwork.h"

#ifdef __GNUC__
// Is this still needed, can someone with gcc 2.95 test without it? :)
#if __GNUC__ < 3
  #include <stdio.h>
#endif
#endif

#ifndef _MSC_VER
#include <strings.h>
#endif

#ifdef _MSC_VER
//#include <hash_map>
#define strcasecmp stricmp
#define strncasecmp strnicmp
#endif

using namespace std;
#if __GNUC__ >= 3
using namespace __gnu_cxx;
#endif

int intCommandPort = 8026;
int intDebugPort = 8016;
#define STDIN 0


gbdNetwork *nwDebug;
gbdNetwork *nwCommand;

int intMessageLevel;
char *chpExecName;

int main(int argc, char** argv)
{

  struct timeval tv;
  fd_set readfds;
  fd_set fdsWait;
  int intCnt;

  // Initial verbosity level
  logOutput.SetVerbosityLevel(LOG_LEVEL_INFO);
  chpExecName = argv[0];

  // Read settings
  ReadSettings();

  // Parse Args
  ParseArgs(argc, argv);

  // Show startup info
  ShowStartupInfo();

  //windows specifics
  //http://www.ecst.csuchico.edu/~beej/guide/net/html/intro.html#windowsv
#if __WIN32__
  WSADATA wsaData;
  //WSAData wsaData;

  if (WSAStartup(MAKEWORD(1, 1), &wsaData) != 0)
  {
    LOG_ERROR "WSAStartup failed.\n";
    Quit(1);
  }
#endif

  // Create the command and debug sockets
  nwCommand = new gbdNetwork(intCommandPort);
  nwDebug = new gbdNetwork(intDebugPort);

  // Set up sockets
  FD_ZERO(&readfds);
#ifndef __WIN32__
  FD_SET(STDIN, &readfds);
#endif
  nwCommand->AddSockets(&readfds);
  nwDebug->AddSockets(&readfds);

  // Loop
  while(1)
  {
    int intRead;

    // Copy original fds
    fdsWait = readfds;

    // Initiate timeout
    tv.tv_sec = 30;
    tv.tv_usec = 0;

    // Wait for input
    if(select(1 + max(nwCommand->GetMaxSocket(), nwDebug->GetMaxSocket()),
              &fdsWait, NULL, NULL, &tv) == -1)
    {
      LOG_ERROR  "Select() error!\n";
      Quit(1);
    }

    if (FD_ISSET(STDIN, &fdsWait))
    {
      string strInput;
      size_t stPos;
      cin >> strInput ;
      ProcessCommand(strInput);
    }

    nwCommand->HandleInput(&fdsWait);
    nwDebug->HandleInput(&fdsWait);
  }
  Quit(0);
  return 0;
}

void Quit(int intError)
{
#if __WIN32__
  WSACleanup();
#endif
  exit(intError);
}

void ProcessAccept(int intPort, bool blConnected)
{

  if(blConnected)
  {
    if(intPort == intDebugPort)
      LOG_INFO "Accepted script on port " << intPort << "\n";
    else
      LOG_INFO "Accepted client on port " << intPort << "\n";
  }

  // If we had a connection already, send the queued buffer
  if(blConnected && intPort == intDebugPort && nwCommand->IsConnected())
    HaveData(intCommandPort);

  if(blConnected && intPort == intCommandPort && nwDebug->IsConnected())
    HaveData(intDebugPort);

  return;
}

// Commands from stdin...
void ProcessCommand(string& strCommand)
{

  LOG_DEBUG "Command: " << strCommand.c_str() << "\n";

  if(strCommand == "quit")
  {
    LOG_VERBOSE strCommand.c_str() << "\n";

    Quit(0);
    return;

  }
  else if(strCommand == "debug")
  {
    LOG_INFO "Enabled debug output\n";

    logOutput.SetVerbosityLevel(LOG_LEVEL_DEBUG);
    return;

  }
  else if(strCommand == "verbose")
  {
    LOG_INFO "Enabled verbose output\n";

    logOutput.SetVerbosityLevel(LOG_LEVEL_VERBOSE);
    return;

  }
  else if(strCommand == "info")
  {
    LOG_INFO "Enabled info output only\n";

    logOutput.SetVerbosityLevel(LOG_LEVEL_INFO);
    return;

  }
  else if(strCommand == "help")
  {
    ShowHelp();
    return;
  }

  LOG_ERROR "Unknown command: \"" << strCommand.c_str() << "\"\n";

}

void HaveData(int port)
{
  if(port == intCommandPort && nwDebug->IsConnected())
  {
    nwDebug->SendData(nwCommand->FlushBuffer());
  }
  if(port == intDebugPort && nwCommand->IsConnected())
  {
    nwCommand->SendData(nwDebug->FlushBuffer());
  }
}

void ShowHelp()
{
  cout << "\n";
  cout << chpExecName << " is a proxy server for the Gubed PHP Debugger for people behind a NAT\n";
  cout << "  or people who wish their PHP-scripts to connect to a predefined IP independent\n";
  cout << "  of who is currently debugging.\n";
  cout << "\n";
  cout << "Available commands: \n";
  cout << "debug                          Enable debug output (shows all data transferred)\n";
  cout << "help                           Quite certain this command does not require further explanation\n";
  cout << "info                           Enable informational output only\n";
  cout << "quit                           Kill the running script (if any) and quit server\n";
  cout << "verbose                        Enable verbose output\n";

}

void ShowArgHelp()
{
  cout << "\n";
  cout << chpExecName << " is a proxy server for the Gubed PHP Debugger for people behind a NAT\n";
  cout << "  or people who wish their PHP-scripts to connect to a predefined IP independent\n";
  cout << "  of who is currently debugging.\n";
  cout << "\n";
  cout << "Available arguments: \n";
  cout << "--debug                         Enable debug output\n";
  cout << "--verbose                       Be verbose\n";
  cout << "--quiet                         Be quiet\n";
  cout << "--commandport <port>            Use port <port> for client\n";
  cout << "--debugport <port>              Use port <port< for script\n";

  cout << "\nConfig file (" << getenv(SETTINGSPATH) << SETTINGSFILE ") is read and parsed first and accepts the same arguments\n";
  cout << "as commandline. Command line arguments override the settings in the configuration file.\n";

}

void ShowStartupInfo()
{
  LOG_INFO "Gubed Proxy v" << PROXYVERSION << " part of Gubed Release " << GUBEDRELEASE << "\n";
  LOG_INFO "Copyright (c) 2004 Linus McCabe\n";
  LOG_INFO "GPL Software, see License.txt\n\n";
  LOG_INFO "Type \"quit\" to shut down the proxy or \"help\" for help\n\n";

  if(logOutput.GetVerbosityLevel() == LOG_LEVEL_DEBUG)
    LOG_INFO "Debug output enabled\n\n";
  else if(logOutput.GetVerbosityLevel() == LOG_LEVEL_VERBOSE)
    LOG_INFO "Verbose output enabled\n\n";

}

void ParseArgs(int argc, char**argv)
{

  for(int intCnt = 1; intCnt < argc; intCnt ++)
  {
    if(intCnt < argc - 1)
    {
      if(ParseConfig(argv[intCnt], argv[intCnt + 1]))
        intCnt ++;
    }
    else
      ParseConfig(argv[intCnt], "");

  }
}

void ReadSettings()
{
  FILE *pfConfig;
  char chBuffer[2047];

  string strFilename;
  strFilename = getenv(SETTINGSPATH);
  strFilename += SETTINGSFILE;

  pfConfig = fopen(strFilename.c_str(), "r");
  if(pfConfig)
  {

    while(!feof(pfConfig))
    {
      if(fgets(chBuffer, 2047, pfConfig))
        if(!ferror(pfConfig))
        {
          chBuffer[strlen(chBuffer) - 1] = 0;
          ParseConfig(chBuffer, "");
        }
    }
  }
}

bool ParseConfig(const char* pchSetting, const char* pchArg)
{

  const char *pchSubStr;
  int intLen = 0;

  pchSubStr = strstr(pchSetting, "=");
  if(!pchSubStr)
  {
    pchSubStr = pchArg;
    intLen = strlen(pchSetting) + 1;
  }
  else
  {
    pchSubStr++;
    intLen = pchSubStr - pchSetting;
  }

  if(intLen <= 1)
    return false;

  LOG_DEBUG "Setting: \"" << pchSetting << "\" (" << intLen << "), possible argument: " << (pchSubStr ? pchSubStr : "") << "\n";

  if(intLen == sizeof("--verbose") && strncasecmp(pchSetting, "--verbose", sizeof("-verbose")) == 0
      || intLen == sizeof("verbose") && strncasecmp(pchSetting, "verbose", sizeof("erbose")) == 0)
  {
    logOutput.SetVerbosityLevel(LOG_LEVEL_VERBOSE);
    return false;
  }

  if(intLen == sizeof("--debug") && strncasecmp(pchSetting, "--debug", sizeof("-debug")) == 0
      || intLen == sizeof("debug") && strncasecmp(pchSetting, "debug", sizeof("ebug")) == 0)
  {
    logOutput.SetVerbosityLevel(LOG_LEVEL_DEBUG);
    return false;
  }

  if(intLen == sizeof("--quiet") && strncasecmp(pchSetting , "--quiet", sizeof("-quiet")) == 0
      || intLen == sizeof("quiet") && strncasecmp(pchSetting , "quiet", sizeof("uiet")) == 0)
  {
    logOutput.SetVerbosityLevel(LOG_LEVEL_ERROR);
    return false;
  }

  if(intLen == sizeof("--help") && strncasecmp(pchSetting , "--help", sizeof("-help")) == 0)
  {
    ShowArgHelp();
    Quit(0);
  }

  if(intLen == sizeof("--commandport") && strncasecmp(pchSetting , "--commandport", sizeof("-commandport")) == 0
      || intLen == sizeof("commandport") && strncasecmp(pchSetting , "commandport", sizeof("ommandport")) == 0)
  {
    if(!pchSubStr)
    {
      LOG_ERROR "commandport: No port specified\n";
      Quit(1);
    }

    intCommandPort = atoi(pchSubStr);
    return true;
  }

  if(intLen == sizeof("--debugport") && strncasecmp(pchSetting , "--debugport", sizeof("-debugport")) == 0
      || intLen == sizeof("debugport") && strncasecmp(pchSetting , "debugport", sizeof("ebugport")) == 0)
  {
    if(!pchSubStr)
    {
      LOG_ERROR "debugport: No port specified\n";
      Quit(1);
    }

    intDebugPort = atoi(pchSubStr);
    return true;
  }

  LOG_ERROR "Unknown argument: \"" << pchSetting << "\" (" << intLen << ")\n";
  ShowArgHelp();
  Quit(1);
}
